/*==============================================================================
	CArrange_Mix.cpp
==============================================================================*/

#include "CArrange.hpp"

/*==============================================================================
	writeProc
==============================================================================*/

int CArrange::writeProc(char *lpBuf)
{

	CSample			*gSample;
	char			tmpBuffer[ _iPreviewBufferSize ];
	unsigned long	iPlayBlockPosition;
	unsigned long	iSamplePosition;
	unsigned int	iSampleLength;
	int				iSampleCount;
	int				i, j;

	// Range les samples par position
	gSampleMap->sortByTimePosition();

	// Compte les samples
	iSampleCount			= gSampleMap->count();

	// Recherche la position
	iPlayBlockPosition		= ( _iPreviewBufferPosition * _iPreviewBufferSize ) / _wFormatPreview.nBlockAlign;

	// Met du silence
	ZeroMemory(lpBuf, _iPreviewBufferSize);

	// Rcupre le son de tous les samples
	for ( i = 0; i < iSampleCount; i++ ) {

		// Infos du samples
		gSample				= gSampleMap->getByIndex(i);

		// Fait le silence
		ZeroMemory(tmpBuffer, _iPreviewBufferSize);

		// Si le sample est dans la fourchette d'octets on le mlange avec lpBuf
		if ( gSample->getBuffer(tmpBuffer, &_wFormatPreview, iPlayBlockPosition, _iPreviewBufferSize) > 0 ) {

			for ( j = 0; j < _iPreviewBufferSize; j++ )
				lpBuf[j]	+= tmpBuffer[j];

		}

	}

	// Actualise la position du curseur (octets / octets par sec.)
	_playPos				= ( 1000 * _iPreviewBufferPosition * _iPreviewBufferSize )
								/ ( _wFormatPreview.nSamplesPerSec * _wFormatPreview.nBlockAlign );
	_iPreviewBufferPosition++;

	// Repeint la table
	rePaint();

	// Retourne le nombre d'octets remplis
	return _iPreviewBufferSize;

}

/*==============================================================================
	stWriteProc
==============================================================================*/

int CArrange::stWriteProc(LPVOID param, char *lpBuf)
{

	CArrange		*pData;

	pData		= (CArrange *)param;
	if ( pData )
		return pData->writeProc(lpBuf);
	return -1;

}

/*==============================================================================
	playMix
==============================================================================*/

void CArrange::playMix(void)
{

	// Efface l'aperu du mix
	stopMix();

	// Configure la sortie audio
	ZeroMemory(&_wFormatPreview, sizeof(_wFormatPreview));
	_wFormatPreview.wFormatTag			= WAVE_FORMAT_PCM;
	_wFormatPreview.nSamplesPerSec		= 44100;
	_wFormatPreview.wBitsPerSample		= 16;
	_wFormatPreview.nChannels			= 2;
	_wFormatPreview.nBlockAlign			= _wFormatPreview.nChannels * ( _wFormatPreview.wBitsPerSample / 8 );
	_wFormatPreview.nAvgBytesPerSec		= _wFormatPreview.nBlockAlign * _wFormatPreview.nSamplesPerSec;

	// Rinitialise la position du curseur
	_iPreviewBufferPosition		= 0;
	_iPreviewBufferSize			= 8 * 1024;

	// Cre un nouvel aperu
	_gWaveOutBuffer		= new CWaveOutBuffer((long)this, stWriteProc,
		_wFormatPreview, _wFormatPreview.nChannels, _iPreviewBufferSize);
	_gWaveOutBuffer->start();

	return;

	/*//stopMix();
	//SetTimer(hWnd, TIMERID_PLAY, 50, 0);

	const int		nTotalDataLength	= 44100 * 4;

	CSample			*gSample;
	WAVEFORMATEX	wfxInput;
	FILE			*fp;
	unsigned long	dwBuf;
	char			*pBuffer;
	unsigned long	iTimeA, iTimeB;
	unsigned long	iBufferPosA, iBufferPosB;
	unsigned long	iBufferSamplePosA, iBufferSamplePosB;
	unsigned long	iTimePos;
	int				iSampleCount, iSampleStart, iSampleEnd;
	int				i, j, k;
	char buf[128];

	// Range les samples dans l'ordre
	gSampleMap->sortByTimePosition();
	rePaint();

	// Initialisation
	iTimePos		= 0;
	iSampleStart	= -1;
	iSampleEnd		= -1;
	iSampleCount	= gSampleMap->count();

	for ( i = 0; i < iSampleCount; i++ ) {

		// Infos du samples
		gSample			= gSampleMap->getByIndex(i);
		iTimeA			= gSample->getTimePosition();
		iTimeB			= gSample->getLength();

		// Si le sample est dans l'intervalle
		if ( iTimeA < iTimePos + 1000 && iTimeA + iTimeB >= iTimePos ) {
			if ( iSampleStart < 0 )
				iSampleStart	= i;
			iSampleEnd		= i + 1;
		}

	}

	// Alloue de la mmoire pour le son
	pBuffer		= new char[ nTotalDataLength ];
	ZeroMemory(pBuffer, nTotalDataLength);
	char		tmpBuffer[ nTotalDataLength ];

	for ( i = iSampleStart; i < iSampleEnd; i++ ) {

		char tmpBuffer[ nTotalDataLength ];

		// Infos du sample
		gSample			= gSampleMap->getByIndex(i);
		gSample->getOneSecBuffer(tmpBuffer, 0);
		for ( j = 0; j < nTotalDataLength; j++ ) {
			pBuffer[j] += tmpBuffer[j];
		}

	}

	ZeroMemory(&wfxInput, sizeof(wfxInput));
    wfxInput.wFormatTag = WAVE_FORMAT_PCM;
    wfxInput.nSamplesPerSec = 44100;
    wfxInput.wBitsPerSample = 16;
    wfxInput.nChannels = 2;
    wfxInput.nBlockAlign = wfxInput.nChannels * (wfxInput.wBitsPerSample / 8);
    wfxInput.nAvgBytesPerSec = wfxInput.nBlockAlign * wfxInput.nSamplesPerSec;

    /*fp = fopen("newwave.wav", "wb");
    if ( !fp )
		MessageBox(gParent->hDlg, "fp == 0", "Info", 0);
	dwBuf = 0x46464952; // RIFF
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = nTotalDataLength + 20 + sizeof(WAVEFORMATEX); // fileSize
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = 0x45564157; // WAVE
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = 0x20746D66; // _fmt
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = sizeof(WAVEFORMATEX); // sizeof(WAVEFORMATEX)
	fwrite(&dwBuf, 4, 1, fp);
	fwrite(&wfxInput, sizeof(wfxInput), 1, fp);
	dwBuf = 0x61746164; // data
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = nTotalDataLength; // waveSize
	fwrite(&dwBuf, 4, 1, fp);
	fwrite(pBuffer, 1, nTotalDataLength, fp);
	fclose(fp);*/

/*
	delete [] pBuffer;

	MessageBox(gParent->hDlg, "Fini", "Info", 0);

	/**===================================================*/

	/*std::list<CSample *>::iterator iter;
	WAVEFORMATEX  wfxInput;
	CSample *sample1, *sample2;
	FILE *fp;
	char *pBuffer;
	char buf1[8], buf2[8];
	long size, size1, size2;
	long i, j;
	unsigned long dwBuf;

    ZeroMemory( &wfxInput, sizeof(wfxInput));
    wfxInput.wFormatTag = WAVE_FORMAT_PCM;
    wfxInput.nSamplesPerSec = 44100;
    wfxInput.wBitsPerSample = 16;
    wfxInput.nChannels = 2;
    wfxInput.nBlockAlign = wfxInput.nChannels * (wfxInput.wBitsPerSample / 8);
    wfxInput.nAvgBytesPerSec = wfxInput.nBlockAlign * wfxInput.nSamplesPerSec;

	iter = gSampleMap->begin();
	sample1 = *iter; iter++;
	sample2 = *iter;

	size1 = sample1->pWaveData->fEndPos - sample1->pWaveData->fStartPos;
	size2 = sample2->pWaveData->fEndPos - sample2->pWaveData->fStartPos;
	if ( size1 >= size2 )
		size = size1;
	else
		size = size2;

	//wfxInput.nSamplesPerSec * wfxInput.wBitsPerSample * 12;

	//sprintf(mybuf, "size : %d", isize);
	//MessageBox(0, mybuf, "Info", 0);
	sprintf(buf1, "%d", size);
	MessageBox(0, buf1, "Taille", 0);
	pBuffer = new char[size+1];
	ZeroMemory(pBuffer, size);
	for ( i = 0; i < size/8; i++ ) {
		sample1->pWaveData->getUncompressedBlock(i, buf1);
		sample2->pWaveData->getUncompressedBlock(i, buf2);
		for ( j = 0; j < 7; j++ )
			pBuffer[i*8+j] = buf1[j] + buf2[j];
	}

	fp = fopen("newwave.wav", "wb");
	dwBuf = 0x46464952; // RIFF
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = size + 20 + sizeof(WAVEFORMATEX); // fileSize
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = 0x45564157; // WAVE
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = 0x20746D66; // _fmt
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = sizeof(WAVEFORMATEX); // sizeof(WAVEFORMATEX)
	fwrite(&dwBuf, 4, 1, fp);
	fwrite(&wfxInput, sizeof(wfxInput), 1, fp);
	dwBuf = 0x61746164; // data
	fwrite(&dwBuf, 4, 1, fp);
	dwBuf = size; // waveSize
	fwrite(&dwBuf, 4, 1, fp);
	fwrite(pBuffer, 1, size, fp);
	fclose(fp);*/

	//pour chaque buffer du son mix
	/*for ( i = 0; i < ; i++)
	{
# //si nous sommes toujours dans le petit son
# if(j<(int)iTaillePS)
# //ajouter les buffers des deux sons
# pWave->m_pData[j] = pDataGS[j] + pDataPS[j];
# else
# //n'ajouter que le buffer du grand son
# pWave->m_pData[j] = pDataGS[j];
# }

	if ( !gSampleMap->empty() ) {
		for ( iter = gSampleMap->begin(); iter != gSampleMap->end(); iter++ ) {
			gSample = *iter;
			if ( gSample->isSelected() ) {
				gSampleMap->erase(iter);
				iter--; // Evite un bug
				delete gSample;
			}
		}
	}*/

	/**===================================================*/

}

/*==============================================================================
	stopMix
==============================================================================*/

void CArrange::stopMix(void)
{

	// Efface l'aperu du mix
	if ( _gWaveOutBuffer ) {
		delete _gWaveOutBuffer;
		_gWaveOutBuffer		= 0;
	}
	_playPos				= 0;
	rePaint();

}

/* Fin de CArrange_Mix.cpp */
