/*==============================================================================
	CArrange_Paint.cpp
==============================================================================*/

#include "CArrange.hpp"

/*==============================================================================
	paintSampleMap
==============================================================================*/

void CArrange::paintSampleMap(HDC tmpHdc, int offsetTime, int visibleTime, int offsetTrack, int visibleTrack)
{

	CSample			*gSample;
	HBRUSH 			hBackground, hSample, hSelectedSample;
	HPEN 			hHorzGrid, hVertGrid, hPlayCursor;
	RECT			sampleMapRect, rc;
	int				i, x;
	char			szText[8];

	// Couleurs et polices
	hBackground 		= CreateSolidBrush(RGB(255, 255, 230)); // Couleur de fond
	hSample 			= CreateSolidBrush(RGB(235, 205, 205)); // Couleur d'un sample
	hSelectedSample 	= CreateSolidBrush(RGB(200, 180, 180)); // Couleur d'un sample slectionn
	hPlayCursor 		= CreatePen(PS_SOLID, 2, RGB(0, 0, 0)); // Couleur du trait de lecture
	hHorzGrid 			= CreatePen(PS_SOLID, 1, RGB(192, 192, 64));
	hVertGrid 			= CreatePen(PS_DOT, 1, RGB(192, 192, 64));
	SelectObject(tmpHdc, (HFONT)GetStockObject(DEFAULT_GUI_FONT)); // Police par dfaut
	SetBkMode(tmpHdc, TRANSPARENT); // Texte sur fond transparent

	// Fond
	sampleMapRect		= getSampleMapRect();
	FillRect(tmpHdc, &sampleMapRect, hBackground);

	// Trace les lignes horizontales
	SelectObject(tmpHdc, hHorzGrid);
	for ( i = 0; i < visibleTrack; i++ ) {
    	x = sampleMapRect.top + i * _trackScale;
		MoveToEx(tmpHdc, sampleMapRect.left, x, (LPPOINT)NULL);
		LineTo(tmpHdc, sampleMapRect.right, x);
	}

	// Trace les lignes verticales
	SelectObject(tmpHdc, hVertGrid);
	for ( i = 0; i < visibleTime; i++ ) {
    	x = sampleMapRect.left + i * _timeScale;
		MoveToEx(tmpHdc, x, sampleMapRect.top, (LPPOINT)NULL);
		LineTo(tmpHdc, x, sampleMapRect.bottom);
	}

	// Trait de lecture
	SelectObject(tmpHdc, hPlayCursor);
	x = sampleMapRect.left + ( _playPos - getOffsetTime() * TIME_PRECISION ) * _timeScale / TIME_PRECISION;
	MoveToEx(tmpHdc, x, sampleMapRect.top, (LPPOINT)NULL);
	LineTo(tmpHdc, x, sampleMapRect.bottom);

	// Samples
	for ( i = 0; i < gSampleMap->count(); i++ ) {
		gSample		= gSampleMap->getByIndex(i);
		if ( gSample != 0 && isSampleVisible(gSample) ) {
			rc		= getSampleRect(gSample);
			if ( gSample->isSelected() )
				FillRect(tmpHdc, &rc, hSelectedSample);
			else
				FillRect(tmpHdc, &rc, hSample);
			char buf[128];
			sprintf(buf, "%d. %u ms (%u ms)", i, gSample->getTimePosition(), gSample->getLength());
			TextOut(tmpHdc, rc.left, rc.top, buf, strlen(buf));
		}
	}

	// Libration de mmoire
	DeleteObject(hBackground);
	DeleteObject(hSample);
	DeleteObject(hSelectedSample);
	DeleteObject(hPlayCursor);
	DeleteObject(hHorzGrid);
	DeleteObject(hVertGrid);


}

/*==============================================================================
	paintTimeBar
==============================================================================*/

void CArrange::paintTimeBar(HDC tmpHdc, int offsetTime, int visibleTime)
{

	HBRUSH hBackground;
	HPEN hTicks;
	RECT timeBarRect;
	int i, x;
	char szText[8];

	// Couleurs et polices
	hBackground		= CreateSolidBrush(RGB(230, 230, 240)); // Couleur de fond
	hTicks			= CreatePen(PS_DOT, 1, RGB(64, 64, 64)); // Ticks
	SelectObject(tmpHdc, hTicks);
	SelectObject(tmpHdc, (HFONT)GetStockObject(DEFAULT_GUI_FONT)); // Police par dfaut
	SetBkMode(tmpHdc, TRANSPARENT); // Texte sur fond transparent

	// Fond
	timeBarRect		= getTimeBarRect();
	FillRect(tmpHdc, &timeBarRect, hBackground);

	// Graduations
    for ( i = 0; i < visibleTime; i++ ) {
    	// Ticks
    	x = timeBarRect.left + i * _timeScale;
		MoveToEx(tmpHdc, x, timeBarRect.bottom - 1, (LPPOINT)NULL);
		LineTo(tmpHdc, x, timeBarRect.bottom - 20);
		// Chiffres
		wsprintf(szText, "%i", offsetTime + i + 1);
		TextOut(tmpHdc, x + _timeScale / 2 - 2, 0, szText, lstrlen(szText));
    }

	// Libration de mmoire
	DeleteObject(hBackground);
	DeleteObject(hTicks);

}

/*==============================================================================
	paintTrackBar
==============================================================================*/

void CArrange::paintTrackBar(HDC tmpHdc, int offsetTrack, int visibleTrack)
{

	HBRUSH hBackground;
	HPEN hTicks;
	RECT trackBarRect;
	int i, y;
	char szText[8];

	// Couleurs et polices
	hBackground		= CreateSolidBrush(RGB(240, 235, 230)); // Couleur de fond
	hTicks			= CreatePen(PS_SOLID, 1, RGB(64, 64, 64)); // Ticks
	SelectObject(tmpHdc, hTicks);
	SelectObject(tmpHdc, (HFONT)GetStockObject(DEFAULT_GUI_FONT)); // Police par dfaut
	SetBkMode(tmpHdc, TRANSPARENT); // Texte sur fond transparent

	// Fond
	trackBarRect	= getTrackBarRect();
	FillRect(tmpHdc, &trackBarRect, hBackground);

	// Graduations
    for ( i = 0; i < visibleTrack; i++ ) {
    	// Ticks
    	y = trackBarRect.top + i * _trackScale;
		MoveToEx(tmpHdc, trackBarRect.left, y, (LPPOINT)NULL);
		LineTo(tmpHdc, trackBarRect.right, y);
		// Chiffres
		wsprintf(szText, "%i", offsetTrack + i + 1);
		TextOut(tmpHdc, trackBarRect.right / 2 - 5, y + _trackScale / 2 - 6, szText, lstrlen(szText));
    }

	// Libration de mmoire
	DeleteObject(hBackground);
	DeleteObject(hTicks);

}

/*==============================================================================
	rePaint
==============================================================================*/

void CArrange::rePaint(void)
{

	InvalidateRect(hWnd, NULL, FALSE);

}


/*==============================================================================
	isSampleVisible
==============================================================================*/

BOOL CArrange::isSampleVisible(CSample *sample)
{

	RECT sampleRect, mixTableRect;

	sampleRect		= getSampleRect(sample);
	mixTableRect	= getSampleMapRect();
	return isRectVisibleInRect(sampleRect, mixTableRect);

}

/*==============================================================================
	paintInt (debug)
==============================================================================*/
/*
void CArrange::paintInt(long n, int x, int y)
{

	HDC tmpHdc;
	HBRUSH hBackground;
	RECT rc;
	char buf[32];

	tmpHdc			= GetDC(hWnd);
	hBackground		= CreateSolidBrush(RGB(200, 200, 200)); // Couleur de fond
	SelectObject(tmpHdc, (HFONT)GetStockObject(DEFAULT_GUI_FONT)); // Police par dfaut
	SetBkMode(tmpHdc, TRANSPARENT);
	rc.left = x;	rc.right = x + 25;
	rc.top = y;		rc.bottom = y + 12;
	FillRect(tmpHdc, &rc, hBackground);
	sprintf(buf, "%d", n);
	TextOut(tmpHdc, x, y, buf, strlen(buf));
	DeleteObject(hBackground);
	ReleaseDC(hWnd, tmpHdc);

}*/

/* Fin de CArrange_Paint.cpp */
