/*==============================================================================
	CArrange_Rect.cpp
==============================================================================*/

#include "CArrange.hpp"

/*==============================================================================
	isPointInRect
==============================================================================*/

BOOL CArrange::isPointInRect(RECT tmpRect, long x, long y)
{

	if ( x >= tmpRect.left && x <= tmpRect.right
		&& y >= tmpRect.top && y <= tmpRect.bottom )
		return 1;
	return 0;

}

/*==============================================================================
	isRectVisibleInRect
==============================================================================*/

BOOL CArrange::isRectVisibleInRect(RECT testRect, RECT tmpRect)
{

	if ( testRect.left <= tmpRect.right && testRect.right >= tmpRect.left &&
		 testRect.top <= tmpRect.bottom && testRect.bottom >= tmpRect.top )
		return 1;
	return 0;

}

/*==============================================================================
	getTimeBarRect
==============================================================================*/

RECT CArrange::getTimeBarRect(void)
{

	RECT tmpRect, timeBarRect;

	tmpRect = getClientRect();
	timeBarRect.left	= _trackBarWidth;
	timeBarRect.right	= tmpRect.right;
	timeBarRect.top		= 0;
	timeBarRect.bottom	= _timeBarHeight;
	return timeBarRect;

}

/*==============================================================================
	getTrackBarRect
==============================================================================*/

RECT CArrange::getTrackBarRect(void)
{

	RECT tmpRect, trackBarRect;

	tmpRect = getClientRect();
	trackBarRect.left	= 0;
	trackBarRect.right	= _trackBarWidth;
	trackBarRect.top	= _timeBarHeight;
	trackBarRect.bottom	= tmpRect.bottom;
	return trackBarRect;

}

/*==============================================================================
	getSampleMapRect
==============================================================================*/

RECT CArrange::getSampleMapRect(void)
{

	RECT tmpRect, mixTableRect;

	tmpRect = getClientRect();
	mixTableRect.left	= _trackBarWidth;
	mixTableRect.right	= tmpRect.right;
	mixTableRect.top	= _timeBarHeight;
	mixTableRect.bottom	= tmpRect.bottom;
	return mixTableRect;

}

/*==============================================================================
	getSampleRect
==============================================================================*/

RECT CArrange::getSampleRect(CSample *sample)
{

	RECT tmpRect;
	unsigned long		samplePos;
	unsigned int		sampleTrack;
	unsigned long		sampleLength;
	unsigned long		offsetTime, visibleTime;
	int					offsetTrack, visibleTrack;

	samplePos		= sample->getTimePosition() * _timeScale / TIME_PRECISION;
	sampleLength	= sample->getLength() * _timeScale / TIME_PRECISION;
	sampleTrack		= sample->getTrack();

	offsetTime		= getOffsetTime();
	visibleTime		= getVisibleTime();
	offsetTrack		= getOffsetTrack();
	visibleTrack	= getVisibleTrack();

	tmpRect.left	= _trackBarWidth + samplePos - offsetTime * _timeScale;
	tmpRect.right	= _trackBarWidth + samplePos - offsetTime * _timeScale + sampleLength;
	tmpRect.top		= _timeBarHeight + ( sampleTrack - offsetTrack ) * _trackScale;
	tmpRect.bottom	= _timeBarHeight + ( sampleTrack - offsetTrack + 1 ) * _trackScale;

	return tmpRect;

}

/*==============================================================================
	getScrollPos
==============================================================================*/

LONG CArrange::getScrollPos(int type)
{

	SCROLLINFO		si;

	ZeroMemory(&si, sizeof(si));
	si.cbSize		= sizeof(si);
	si.fMask		= SIF_POS;
	GetScrollInfo(hWnd, type, &si);
	return si.nPos;

}

/*==============================================================================
	moveScrollPos
==============================================================================*/

void CArrange::moveScrollPos(int type, int iOffset)
{

	SCROLLINFO		si;

	ZeroMemory(&si, sizeof(si));
	si.cbSize		= sizeof(si);
	si.fMask		= SIF_POS;
	GetScrollInfo(hWnd, type, &si);
	si.nPos			= si.nPos + iOffset;
	SetScrollInfo(hWnd, type, &si, TRUE);

}

/*==============================================================================
	isInLeftMoveRect
==============================================================================*/

BOOL CArrange::isInLeftMoveRect(int xPos, int yPos)
{

	RECT		rc;

	rc			= getClientRect();
	rc.left		= 0;
	rc.right	= MOVE_RECT_SIZE + _trackBarWidth;

	return ( isPointInRect(rc, xPos, yPos) );

}

/*==============================================================================
	isInRightMoveRect
==============================================================================*/

BOOL CArrange::isInRightMoveRect(int xPos, int yPos)
{

	RECT		rc;

	rc			= getClientRect();
	rc.left		= rc.right - MOVE_RECT_SIZE;

	return ( isPointInRect(rc, xPos, yPos) );

}

/*==============================================================================
	isInUpMoveRect
==============================================================================*/

BOOL CArrange::isInUpMoveRect(int xPos, int yPos)
{

	RECT		rc;

	rc			= getClientRect();
	rc.top		= 0;
	rc.bottom	= MOVE_RECT_SIZE + _timeBarHeight;

	return ( isPointInRect(rc, xPos, yPos) );

}

/*==============================================================================
	isInDownMoveRect
==============================================================================*/

BOOL CArrange::isInDownMoveRect(int xPos, int yPos)
{

	RECT		rc;

	rc			= getClientRect();
	rc.top		= rc.bottom - MOVE_RECT_SIZE;

	return ( isPointInRect(rc, xPos, yPos) );

}

/*==============================================================================
	getOffsetTime
==============================================================================*/

unsigned long CArrange::getOffsetTime(void)
{

	return getScrollPos(SB_HORZ);

}

/*==============================================================================
	getOffsetTrack
==============================================================================*/

int CArrange::getOffsetTrack(void)
{

	return getScrollPos(SB_VERT);

}

/*==============================================================================
	getVisibleTime
==============================================================================*/

int CArrange::getVisibleTime(void)
{

	return ( ( getClientWidth() - _trackBarWidth ) / _timeScale + 1 );

}

/*==============================================================================
	getVisibleTrack
==============================================================================*/

int CArrange::getVisibleTrack(void)
{

	return ( ( getClientHeight() - _timeBarHeight ) / _trackScale + 1 );

}

/* Fin de CArrange_Rect.cpp */
