/*==============================================================================
	CArrange_Sample.cpp
==============================================================================*/

#include "CArrange.hpp"

/*==============================================================================
	addSample
==============================================================================*/

CSample *CArrange::addSample(CSample *gSample)
{

	gSampleMap->add(gSample);
	return gSample;

}

/*==============================================================================
	selectSample
==============================================================================*/

CSample *CArrange::selectSample(int xPos, int yPos)
{

	RECT			sampleRect;
	CSample			*gSample;
	int				i;

	// Cherche le premier sample slectionn
	for ( i = 0; i < gSampleMap->count(); i++ ) {
		gSample					= gSampleMap->getByIndex(i);
		sampleRect				= getSampleRect(gSample);
		if ( isPointInRect(sampleRect, xPos, yPos) ) {
			gSample->pointerX	= xPos - sampleRect.left;
			gSample->pointerY	= yPos - sampleRect.top;
			setSelectedSample(gSample);
			return gSample;
		}
	}
	return 0;

}

/*==============================================================================
	moveSample
==============================================================================*/

void CArrange::moveSample(CSample *gSample, int xPos, int yPos)
{

	RECT			tmpRect;
	int				samplePosx, samplePosy;
	long long		relativeTime;
	unsigned long	sampleTime;
	unsigned int	sampleTrack;

	// Rcupre les dimensions de la table
	tmpRect			= getSampleMapRect();

	// Rectifie la position du curseur si en dehors
	if ( xPos < tmpRect.left )
		xPos = tmpRect.left;
	if ( xPos > tmpRect.right )
		xPos = tmpRect.right;
	if ( yPos < tmpRect.top )
		yPos = tmpRect.top;
	if ( yPos > tmpRect.bottom )
		yPos = tmpRect.bottom;

	// Rcupre la position gomtrique du sample
	samplePosx 		= xPos - gSample->pointerX - tmpRect.left;
	samplePosy 		= yPos - tmpRect.top;

	// Temps par rapport  l'origine
	relativeTime	= (long)( getOffsetTime() * TIME_PRECISION ) + (long)( samplePosx * TIME_PRECISION / _timeScale );

	// Modifie les donnes du sample
	if ( relativeTime > 0 )
		sampleTime 		= relativeTime;
	else
		sampleTime 		= 0;
	sampleTrack 	= getOffsetTrack() + samplePosy / _trackScale;

	// Affecte le sample
	gSample->setTimePosition(sampleTime);
	gSample->setTrack(sampleTrack);

}

/*==============================================================================
	unselectAllSamples
==============================================================================*/

void CArrange::unselectAllSamples(void)
{

	CSample		*gSample;
	int			i;

	// Dselectionne les samples un par un
	for ( i = 0; i < gSampleMap->count(); i++ ) {
		gSample		= gSampleMap->getByIndex(i);
		gSample->unselect();
	}

}

/*==============================================================================
	getSelectedSample
==============================================================================*/

CSample *CArrange::getSelectedSample(void)
{

	CSample		*gSample;
	int			i;

	// Retrouve le premier sample slectionn
	for ( i = 0; i < gSampleMap->count(); i++ ) {
		gSample		= gSampleMap->getByIndex(i);
		if ( gSample->isSelected() )
			return gSample;
	}
	return 0;

}

/*==============================================================================
	setSelectedSample
==============================================================================*/

void CArrange::setSelectedSample(CSample *gSample)
{

	// Slectionne un sample et dselectionen les autres
	unselectAllSamples();
	gSample->select();

}

/*==============================================================================
	deleteSelectedSamples
==============================================================================*/

void CArrange::deleteSelectedSamples(void)
{

	CSample		*gSample;
	int			i;

	// Retrouve les samples slectionns et les supprime
	for ( i = 0; i < gSampleMap->count(); i++ ) {
		gSample		= gSampleMap->getByIndex(i);
		if ( gSample->isSelected() ) {
			gSampleMap->remove(gSample);
			i--;
		}
	}

}

/*==============================================================================
	deleteAllSamples
==============================================================================*/

void CArrange::deleteAllSamples(void)
{

	gSampleMap->clear();

}

/* Fin de CArrange_Sample.cpp */
