/*==============================================================================
	CArrange_WndProc.cpp
==============================================================================*/

#include "CArrange.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CArrange::CArrange(CMainDialog *parent) :
	_gWaveOutBuffer(0),
	_gPreview(0), _bSampleState(0), _playPos(0),
	_timeBarHeight(30),
	_trackBarWidth(40),
	_timeScale(60),
	_trackScale(40),
	_isScrollingHorz(0), _isScrollingVert(0)
{

	WNDCLASSEX		wc;

	// Rcupre l'adresse du parent
	gParent = parent;

	// Cre une map de samples
	gSampleMap			= new CSampleList();

	// Proprits
	dwStyle 			= WS_HSCROLL|WS_VSCROLL|WS_VISIBLE|WS_CHILD;
	dwStyleEx 			= WS_EX_STATICEDGE;
	wId					= IDC_ARRANGE;

	// Enregistre la classe
	ZeroMemory(&wc, sizeof(wc));
	wc.cbSize			= sizeof(wc);
	wc.style			= CS_DBLCLKS|CS_HREDRAW|CS_VREDRAW;
	wc.hInstance		= gParent->getInstance();
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.lpszClassName	= "SWM_CArrange_WndClass";
	registerClass(&wc);

	// Ouvre le gadget
	open(gParent->getInstance(), gParent->hDlg);

}

/*==============================================================================
	onCreate
==============================================================================*/

void CArrange::onCreate(LPARAM lParam)
{

	SCROLLINFO		si;

	// Rgle les scrollbar
	ZeroMemory(&si, sizeof(si));
	si.cbSize		= sizeof(si);
	si.fMask		= SIF_PAGE|SIF_POS|SIF_RANGE|SIF_TRACKPOS;
	si.nMin			= 0;
	si.nMax			= 200;
	si.nPage		= 10;
	si.nPos			= 0;
	si.nTrackPos	= 0;
	SetScrollInfo(hWnd, SB_HORZ, &si, TRUE); // Horizontal
	si.nMax			= 100;
	si.nPage		= 10;
	SetScrollInfo(hWnd, SB_VERT, &si, TRUE); // Vertical

}

/*==============================================================================
	onScroll
==============================================================================*/

void CArrange::onScroll(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	SCROLLINFO		si;
	int				type;
	int				p;

	// Horizontal ou vertical ?
	if ( uMsg == WM_HSCROLL )
		type = SB_HORZ;
	else
		type = SB_VERT;

	// Rcupre les informations sur le changement de position de la scrollbar
	ZeroMemory(&si, sizeof(si));
	si.cbSize = sizeof(si);
	si.fMask = SIF_PAGE|SIF_POS|SIF_RANGE|SIF_TRACKPOS;
	GetScrollInfo(hWnd, type, &si);

	// Position du curseur
	p = si.nPos;

	switch( LOWORD(wParam) ) {
		case SB_LINEUP:
			p -= 1;
			break;
		case SB_LINEDOWN:
			p += 1;
			break;
		case SB_PAGEUP:
			p -= si.nPage;
			break;
		case SB_PAGEDOWN:
			p += si.nPage;
			break;
		case SB_THUMBPOSITION:
			p = si.nPos;
			break;
		case SB_THUMBTRACK:
			p = si.nTrackPos;
			break;
		default:
			return;
	}

	// Evite les dbordements
	if ( p < si.nMin )
		p = si.nMin;
	if ( p > si.nMax )
		p = si.nMax;

	// Met  jour l'affichage
	SetScrollPos(hWnd, type, p, TRUE);
	rePaint();

}

/*==============================================================================
	onPaint
==============================================================================*/

void CArrange::onPaint(void)
{

	PAINTSTRUCT		ps;
	HDC				tmpHdc;
	HDC				memDC;
	HBITMAP			memBMP;
	RECT			tmpRect, rc;
	int				offsetTime, offsetTrack;
	int				visibleTime, visibleTrack;

	// Rcupre les dimensions du gadget
	tmpRect			= getClientRect();

	// Calcule la visibilit du temps
	offsetTime		= getOffsetTime();
	visibleTime		= getVisibleTime();

	// Calcule la visibilit des pistes
	offsetTrack		= getOffsetTrack();
	visibleTrack	= getVisibleTrack();

	// Commence le dessin
	tmpHdc			= BeginPaint(hWnd, &ps);

	// Alloue de la mmoire pour un bitmap
	memDC			= CreateCompatibleDC(tmpHdc);
	memBMP			= CreateCompatibleBitmap(tmpHdc, tmpRect.right, tmpRect.bottom );
	SelectObject(memDC, memBMP);

	// Peint le carr en haut  gauche (pour win98)
	rc				= getTimeBarRect();
	rc.right		= rc.left;
	rc.left			= 0;
	FillRect(memDC, &rc, (HBRUSH)GetStockObject(BLACK_BRUSH));

	// Peint les trois lments
	paintSampleMap(memDC, offsetTime, visibleTime, offsetTrack, visibleTrack);
	paintTimeBar(memDC, offsetTime, visibleTime);
	paintTrackBar(memDC, offsetTrack, visibleTrack);

	// Copie le bitmap dans la zone client
	BitBlt(tmpHdc, 0, 0, tmpRect.right, tmpRect.bottom, memDC, 0, 0, SRCCOPY);

	// Libre la mmoire
	DeleteObject(memDC);
	DeleteObject(memBMP);
	EndPaint(hWnd, &ps);

}

/*==============================================================================
	onLeftButtonDown
==============================================================================*/

void CArrange::onLeftButtonDown(WPARAM wParam, LPARAM lParam)
{

	CSample		*gSample;
	int			xPos, yPos;

	// Rcupre les coordonnes du curseur
	xPos = LOWORD(lParam);
	yPos = HIWORD(lParam);

	// Bouton gauche cliqu
	if ( wParam & MK_LBUTTON ) {

		// Dselectionne tous les samples
		unselectAllSamples();

		// Dtecte si le curseur est dans la table de mixage et non dans les barres
		if ( isPointInRect(getSampleMapRect(), xPos, yPos) ) {

			// Trouve le sample qui est sous le curseur
			gSample = selectSample(xPos, yPos);

			// Slectionne le sample
			if ( gSample != 0 ) {
				setSelectedSample(gSample);
				_bSampleState		= SAMPLE_STATE_MOVING;
			}

		}

		// Repeint la table
		rePaint();

	}

}

/*==============================================================================
	onRightButtonDown
==============================================================================*/

void CArrange::onRightButtonDown(WPARAM wParam, LPARAM lParam)
{

	CSample		*gSample;
	POINT		iPoint;
	int			xPos, yPos;

	// Rcupre les coordonnes du curseur
	xPos		= LOWORD(lParam);
	yPos		= HIWORD(lParam);

	// Trouve la position absolue du curseur
	iPoint.x	= xPos;
	iPoint.y	= yPos;
	ClientToScreen(hWnd, &iPoint);

	// Bouton droit cliqu
	if ( wParam & MK_RBUTTON ) {

		// Dselectionne tous les samples
		unselectAllSamples();

		// Dtecte si le curseur est dans la table de mixage et non dans les barres
		if ( isPointInRect(getSampleMapRect(), xPos, yPos) ) {

			// Trouve un sample qui est sous le curseur
			gSample = selectSample(xPos, yPos);

			// Sample dtect
			if ( gSample != 0 ) {

				// Slectionne le sample et affiche le menu
				setSelectedSample(gSample);
				rePaint();
				showPopupMenu(IDR_SAMPLE, iPoint.x, iPoint.y);

			} else {

				// Affiche le menu
				rePaint();
				showPopupMenu(IDR_SAMPLEMAP, iPoint.x, iPoint.y);

			}

		}

	}

}

/*==============================================================================
	onMouseMove
==============================================================================*/

void CArrange::onMouseMove(WPARAM wParam, LPARAM lParam)
{

	CSample		*gSample;
	int			xPos, yPos;

	// Evite les mouvements non-dsirs
	if ( _bSampleState == 0 )
		return;

	// Rcupre les coordonnes du curseur
	xPos			= LOWORD(lParam);
	yPos			= HIWORD(lParam);

	// Rcupre le premier sample qui se trouve sous la souris
	gSample			= getSelectedSample();

	// S'il y en a un
	if ( gSample != 0 && wParam & MK_LBUTTON ) {

		// Dplace le sample
		moveSample(gSample, xPos, yPos);

		// Dtecte si la souris se trouve dans la zone de dplacement gauche/droite
		if ( _isScrollingHorz != -1 && isInLeftMoveRect(xPos, yPos) ) {
			_isScrollingHorz		= -1;
			SetTimer(hWnd, TIMERID_MOVE, TIMERDELAY_MOVE, NULL);
		} else if ( _isScrollingHorz != 1 && isInRightMoveRect(xPos, yPos) ) {
			_isScrollingHorz		= 1;
			SetTimer(hWnd, TIMERID_MOVE, TIMERDELAY_MOVE, NULL);
		}

		// Dtecte si la souris se trouve dans la zone de dplacement haut/bas
		if ( _isScrollingVert != -1 && isInUpMoveRect(xPos, yPos) ) {
			_isScrollingVert		= -1;
			SetTimer(hWnd, TIMERID_MOVE, TIMERDELAY_MOVE, NULL);
		} else if ( _isScrollingVert != 1 && isInDownMoveRect(xPos, yPos) ) {
			_isScrollingVert		= 1;
			SetTimer(hWnd, TIMERID_MOVE, TIMERDELAY_MOVE, NULL);
		}

		// Actualise
		SetCursor(LoadCursor(NULL, IDC_HAND));
		rePaint();

	}

}

/*==============================================================================
	onLeftButtonUp
==============================================================================*/

void CArrange::onLeftButtonUp(WPARAM wParam, LPARAM lParam)
{

	// Dsactive les mouvements quand le bouton de la souris est lch
	_bSampleState			= 0;

}

/*==============================================================================
	onLeftButtonDoubleClick
==============================================================================*/

void CArrange::onLeftButtonDoubleClick(WPARAM wParam, LPARAM lParam)
{

	CSample		*gSample;
	int			xPos, yPos;

	// Rcupre les coordonnes du curseur
	xPos = LOWORD(lParam);
	yPos = HIWORD(lParam);

	// Dselectionne tous les samples
	unselectAllSamples();

	// Dtecte si le curseur est dans la table de mixage et non dans les barres
	if ( isPointInRect(getSampleMapRect(), xPos, yPos) ) {

		// Trouve le sample qui est sous le curseur
		gSample = selectSample(xPos, yPos);

		// Slectionne le sample
		if ( gSample != 0 ) {

			// Stoppe la prvisualisation
			if ( _gPreview )
				_gPreview->stop();

			// Slectionne le sample
			setSelectedSample(gSample);

			// Joue le sample
			_gPreview		= gSample;
			_gPreview->play();

		}

	}

	// Repeint la table
	rePaint();

}

/*==============================================================================
	onDragDrop
==============================================================================*/

void CArrange::onDragDrop(WPARAM wParam, LPARAM lParam)
{

	CSample			*gSample;
	CSample			*gDropSample;

	// Pointeur vers un sample
	gDropSample		= (CSample *)lParam;

	// Cre un nouveau sample et l'ajoute  la liste
	gSample			= new CSample(gDropSample->lpFileName);
	addSample(gSample);

	// Actualise la table de mixage
	moveSample(gSample, (short)LOWORD(wParam), (short)HIWORD(wParam));
	rePaint();

}

/*==============================================================================
	onTimer
==============================================================================*/

void CArrange::onTimer(WPARAM wParam, LPARAM lParam)
{

	CSample			*gSample;
	POINT			lpMouse;

	// Rcupre la position du curseur par rapport  la table
	GetCursorPos(&lpMouse);
	ScreenToClient(hWnd, &lpMouse);

	// Rcupre le sample slectionn
	gSample		= getSelectedSample();

	switch ( wParam ) {

		case TIMERID_MOVE:

			// Si le sample n'est plus en dplacement annule le timer
			if ( !gSample || _bSampleState != SAMPLE_STATE_MOVING ) {
				_isScrollingHorz		= 0;
				_isScrollingVert		= 0;
				KillTimer(hWnd, wParam);
				break;
			}

			// Mouvement horizontal
			if ( _isScrollingHorz != 0 && ( isInLeftMoveRect(lpMouse.x, lpMouse.y) || isInRightMoveRect(lpMouse.x, lpMouse.y) ) )
				moveScrollPos(SB_HORZ, _isScrollingHorz);

			// Mouvement vertical
			if ( _isScrollingVert != 0 && ( isInUpMoveRect(lpMouse.x, lpMouse.y) || isInDownMoveRect(lpMouse.x, lpMouse.y) ) )
				moveScrollPos(SB_VERT, _isScrollingVert);

			// Acutalise la position
			moveSample(gSample, lpMouse.x, lpMouse.y);
			rePaint();
			break;

	}

}

/*==============================================================================
	onCommand
==============================================================================*/

void CArrange::onCommand(WPARAM wParam, LPARAM lParam)
{

	int bInt;

	switch ( LOWORD(wParam) ) {

		case IDM_SAMPLE_MOVE:
			MessageBox(gParent->hDlg, "Pas encore implment", "Attention", MB_ICONWARNING);
			break;

		case IDM_SAMPLE_DELETE:
			deleteSelectedSamples();
			rePaint();
			break;

		case IDM_SAMPLEMAP_DELETEALL:
			bInt = MessageBox(gParent->hDlg, "Tout supprimer ?", "Confirmation", MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
			if ( bInt == IDYES ) {
				deleteAllSamples();
				rePaint();
			}
			break;

		case IDM_TEST_:
			gSampleMap->sortByTimePosition();
			rePaint();
			break;

	}

}

/*==============================================================================
	gadgetProc
==============================================================================*/

int CArrange::gadgetProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch ( uMsg ) {

		case WM_CREATE:
			onCreate(lParam);
			break;

		case WM_HSCROLL:
		case WM_VSCROLL:
			onScroll(uMsg, wParam, lParam);
			break;

		case WM_PAINT:
			onPaint();
			break;

		case WM_LBUTTONDOWN:
			onLeftButtonDown(wParam, lParam);
			break;

		case WM_RBUTTONDOWN:
            onRightButtonDown(wParam, lParam);
            break;

		case WM_MOUSEMOVE:
			onMouseMove(wParam, lParam);
			break;

		case WM_LBUTTONUP:
			onLeftButtonUp(wParam, lParam);
			break;

		case WM_LBUTTONDBLCLK:
			onLeftButtonDoubleClick(wParam, lParam);
			break;

		case WM_USER+1:
			onDragDrop(wParam, lParam);
			break;

		case WM_TIMER:
			onTimer(wParam, lParam);
			break;

		case WM_DESTROY:
			unregisterClass();
			delete gSampleMap;
			break;

		case WM_COMMAND:
			onCommand(wParam, lParam);
			break;

		default:
			return 0;

	}
	return 1;

}

/* Fin de CArrange_WndProc.cpp */
