/*==============================================================================
	CDialog.cpp
==============================================================================*/

#include "CDialog.hpp"

/*==============================================================================
	Destructeur
==============================================================================*/

CDialog::~CDialog()
{

	close();

}

/*==============================================================================
	initControls
==============================================================================*/

BOOL CDialog::initControls(DWORD dwICC)
{

	INITCOMMONCONTROLSEX	lpInitCtrls;
	lpInitCtrls.dwSize = sizeof(lpInitCtrls);
	lpInitCtrls.dwICC = dwICC;
	return ::InitCommonControlsEx(&lpInitCtrls);

}

/*==============================================================================
	open
==============================================================================*/

BOOL CDialog::open(HINSTANCE hInstance, HWND hWndParent)
{

	return ::DialogBoxParam(hInstance, MAKEINTRESOURCE(_wTemplate), hWndParent,
		(DLGPROC)stDialogProc, (LPARAM)this);

}

/*==============================================================================
	close
==============================================================================*/

BOOL CDialog::close(void)
{

	if ( hDlg && ::EndDialog(hDlg, 0) ) {
		hDlg = NULL;
		return 1;
	}
	return 0;

}

/*==============================================================================
	setMenu
==============================================================================*/

BOOL CDialog::setMenu(WORD wMenu)
{

	HMENU		hMenu;
	if ( hDlg ) {
		hMenu = LoadMenu((HINSTANCE)GetWindowLong(hDlg, GWL_HINSTANCE), MAKEINTRESOURCE(wMenu));
		return SetMenu(hDlg, hMenu);
	}
	return 0;

}

/*==============================================================================
	getInstance
==============================================================================*/

HINSTANCE CDialog::getInstance(void)
{

	HINSTANCE	hInstance;
	hInstance = (HINSTANCE)GetWindowLong(hDlg, GWL_HINSTANCE);
	return hInstance;

}

/*==============================================================================
	setIcon
==============================================================================*/

void CDialog::setIcon(WORD iIcon)
{

	HICON 		hIcon;
	hIcon = (HICON)LoadImage(getInstance(), MAKEINTRESOURCE(iIcon), IMAGE_ICON, 16, 16, 0);
	SendMessage(hDlg, WM_SETICON, ICON_SMALL, (LPARAM)hIcon);

}

/*==============================================================================
	getUserData
==============================================================================*/

LONG CDialog::getUserData(void)
{

	LONG		lData;
	lData = GetWindowLong(hDlg, GWL_USERDATA);
	return lData;

}

/*==============================================================================
	setUserData
==============================================================================*/

void CDialog::setUserData(LONG lData)
{

	SetWindowLong(hDlg, GWL_USERDATA, lData);

}

/*==============================================================================
	getClientWidth
==============================================================================*/

LONG CDialog::getClientWidth(void)
{

	RECT		lpRect;
	GetClientRect(hDlg, &lpRect);
	return ( lpRect.right - lpRect.left );

}

/*==============================================================================
	getClientHeight
==============================================================================*/

LONG CDialog::getClientHeight(void)
{

	RECT		lpRect;
	GetClientRect(hDlg, &lpRect);
	return ( lpRect.bottom - lpRect.top );

}

/*==============================================================================
	stDialogProc
==============================================================================*/

INT_PTR CALLBACK CDialog::stDialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	CDialog		*pDialog;

	if ( uMsg == WM_INITDIALOG ) {

		::SetWindowLong(hWnd, GWL_USERDATA, lParam);
		pDialog = (CDialog *)lParam;

		if ( pDialog != NULL )
			pDialog->hDlg = hWnd;

	} else {

		pDialog = (CDialog *)::GetWindowLong(hWnd, GWL_USERDATA);

	}

	if ( pDialog != NULL )
		return pDialog->dialogProc(uMsg, wParam, lParam);

	return 0;

}

/*==============================================================================
	dialogProc
==============================================================================*/

INT_PTR CDialog::dialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	if ( uMsg == WM_CLOSE )
		close();
	return 0;

}

/* Fin de CDialog.cpp */
