/*==============================================================================
	CMainDialog.cpp
==============================================================================*/

#include "CMainDialog.hpp"			// Fentre principale
#include "../resources/resource.h"	// ID ressources

/*==============================================================================
	Constructeur
==============================================================================*/

CMainDialog::CMainDialog(HINSTANCE hInstance)
{

	// Applique l'id correspondant
	_wTemplate		= IDD_DIALOG;

	// Initialise les contrles
	initControls(ICC_PROGRESS_CLASS|ICC_LISTVIEW_CLASSES|ICC_BAR_CLASSES|ICC_UPDOWN_CLASS);

	// Ouvre la fentre
	open(hInstance);

}

/*==============================================================================
	onInitDialog
==============================================================================*/

void CMainDialog::onInitDialog(LPARAM lParam)
{

	HICON		hIcon;
	RECT		rc;

	// Fabrique la toolbar
	onMakingToolbar();

	// Fabrique le mixeur
	gArrangeGadget 		= new CArrange(this);
	gSampleCollection 	= new CSampleCollection(this);

	// Fabrique le splitter
	gSplitter 			= new CSplitter(hDlg, IDC_SPLITTER,
							gSampleCollection->hWnd, gArrangeGadget->hWnd, 180, TRUE);
	gSplitter->setMinimumSizes(10, 10);

	// Affiche le menu de la fentre principale
	setMenu(IDR_MAIN_DIALOG);

	// Applique l'icne
	setIcon(IDI_MAIN);

}

/*==============================================================================
	onMakingToolbar
==============================================================================*/

void CMainDialog::onMakingToolbar(void)
{

	const int		tbMaxButtons = 6;
	const int		iImgListID = 0;
	TBADDBITMAP		tbBmp;
	HIMAGELIST		hImageList;
	int				bitmapSX, bitmapSY;
	int				iBitmapResource;

	// Rcupre la profondeur de couleur (32-bit ou autre)
	if ( GetDeviceCaps(GetDC(GetDesktopWindow()), BITSPIXEL) >= 32 )
		iBitmapResource		= IDB_MAINTOOLBAR_32;
	else
		iBitmapResource		= IDB_MAINTOOLBAR_16;

	// Attribue un bitmap
	hImageList				= ImageList_LoadImage(getInstance(), MAKEINTRESOURCE(iBitmapResource),
									32, 0, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION);
	SendDlgItemMessage(hDlg, IDC_MAINTOOLBAR, TB_SETIMAGELIST, (WPARAM)iImgListID, (LPARAM)hImageList);

	// Cre la liste de boutons
	TBBUTTON tbButtons[tbMaxButtons] = {

		{ MAKELONG(0, iImgListID), IDM_NEW, TBSTATE_ENABLED,
			TBSTYLE_BUTTON, {0}, 0, (INT_PTR)"Nouveau" },

		{ MAKELONG(1, iImgListID), IDM_OPEN, TBSTATE_ENABLED,
			TBSTYLE_BUTTON, {0}, 0, (INT_PTR)"Ouvrir" },

		{ MAKELONG(2, iImgListID), IDM_SAVE, TBSTATE_ENABLED,
			TBSTYLE_BUTTON, {0}, 0, (INT_PTR)"Enregistrer" },

		{ 0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, {0}, 0, 0 },

		{ MAKELONG(3, iImgListID), IDM_PLAYMIX, TBSTATE_ENABLED,
			TBSTYLE_BUTTON, {0}, 0, (INT_PTR)"Jouer" },

		{ MAKELONG(4, iImgListID), IDM_STOPMIX, TBSTATE_ENABLED,
			TBSTYLE_BUTTON, {0}, 0, (INT_PTR)"Stop" }

	};

	// Ajoute les boutons
	SendDlgItemMessage(hDlg, IDC_MAINTOOLBAR, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);
    SendDlgItemMessage(hDlg, IDC_MAINTOOLBAR, TB_ADDBUTTONS, (WPARAM)tbMaxButtons, (LPARAM)&tbButtons);

    // Ajuste la barre d'outils
	SendDlgItemMessage(hDlg, IDC_MAINTOOLBAR, TB_AUTOSIZE, 0, 0);

}

/*==============================================================================
	onCommand
==============================================================================*/

void CMainDialog::onCommand(WPARAM wParam, LPARAM lParam)
{

	int					bInt;

	switch ( LOWORD(wParam) ) {

		case IDM_OPEN:
			break;

		case IDM_PLAYMIX:
			gArrangeGadget->playMix();
			break;

		case IDM_STOPMIX:
			gArrangeGadget->stopMix();
			break;

		case IDM_ABOUT:
			MessageBox(hDlg, "Mixeur audio v0.0.1 par ordiman85", "A propos...", MB_ICONINFORMATION);
			break;

		case IDM_QUIT:
			bInt = MessageBox(hDlg, "Quitter ?", "Confirmation", MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2);
			if ( bInt == IDYES )
				close();
			break;

	}

}

/*==============================================================================
	onSize
==============================================================================*/

void CMainDialog::onSize(WPARAM wParam, LPARAM lParam)
{

	LONG		dialogWidth, dialogHeight, toolbarHeight;
	RECT		rc;

	// Rcupre les longueurs
	dialogWidth			= getClientWidth();
	dialogHeight		= getClientHeight();

	// Rcupre la hauteur de la toolbar
	GetWindowRect(GetDlgItem(hDlg, IDC_MAINTOOLBAR), &rc);
	toolbarHeight		= rc.bottom - rc.top;

	// Dplace les lments
	MoveWindow(GetDlgItem(hDlg, IDC_SPLITTER), 0, toolbarHeight, dialogWidth, dialogHeight-toolbarHeight, TRUE);

}

/*==============================================================================
	dialogProc
==============================================================================*/

INT_PTR CMainDialog::dialogProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch ( uMsg ) {

		case WM_INITDIALOG:
			onInitDialog(lParam);
			break;

		case WM_COMMAND:
			onCommand(wParam, lParam);
			break;

		case WM_SIZE:
			onSize(wParam, lParam);
			break;

		case WM_NOTIFY:
			gSampleCollection->onNotify(hDlg, lParam);
			break;

		case WM_MOUSEMOVE:
			gSampleCollection->moveDrag(hDlg, lParam);
			break;

		case WM_LBUTTONUP:
			gSampleCollection->endDrag(hDlg, lParam);
			break;

		case WM_CLOSE:
			delete gArrangeGadget;
			delete gSampleCollection;
			close();
			break;

		default:
			return 0;

	}
	return 1;

}

/* Fin de CMainDialog.cpp */
