/*==============================================================================
	CResampler.hpp

	cresampler.cpp
		http://www.koders.com/cpp/fid9A6E288A6F62B57DA6007AEF571EE888B0BEDA9E.aspx
	cfloatresampler.cpp
		http://www.koders.com/cpp/fidE925A6C081FA496C299C079882AF75B2387102BD.aspx
	cplayloop.cpp
		http://www.koders.com/cpp/fid06890AA31392CFB2EB1AF58CCB67936442F13B50.aspx
	cringbuffer.cpp
		http://www.koders.com/cpp/fid506E8D3B09B3308E9580241AF2B372C777F0E995.aspx

	Y = RESAMPLE(X,P,Q)
		X : signal
		P : frquence destination
		Q : frquence origine
		Y : p/q*Fs

	http://en.wikipedia.org/wiki/Pulse-code_modulation
	http://fr.wikipedia.org/wiki/%C3%89chantillonnage_%28signal%29
	http://fr.wikipedia.org/wiki/Th%C3%A9or%C3%A8me_d'%C3%A9chantillonnage_de_Nyquist-Shannon
	http://en.wikipedia.org/wiki/Nyquist%E2%80%93Shannon_sampling_theorem
	http://en.wikipedia.org/wiki/Lanczos_resampling

	Bilinear Resample Algorithm
		http://www.mediamacros.com/item/item-1006687607/
	Audio File Sample Rate Conversion
		http://www.leute.server.de/wilde/resample.html

	ACM Conversion :
		http://msdn2.microsoft.com/en-us/library/ms705084(VS.85).aspx
		http://www.experts-exchange.com/Microsoft/Development/.NET/Visual_CPP/Q_21937565.html
==============================================================================*/

#ifndef __CRESAMPLER__HPP__
#define __CRESAMPLER__HPP__

#include "../windowsapi.h"

/*==============================================================================
	Classe
==============================================================================*/

class CResampler
{

	public:

		CResampler(WAVEFORMATEX*, char*, unsigned int);

		// Traitement
		char				*process(WAVEFORMATEX*, char*, unsigned int);
		void				free(void);

	protected:

		typedef struct {
			WAVEFORMATEX	wFx;
			char			*pBuf;
			unsigned int	size;
		} WAVEBUFFER;

		WAVEBUFFER			_lpSource;
		WAVEBUFFER			_lpDestination;

		// Canaux
		void				monoToStereo(void);

		// Mmoire
		void				initBuffer(WAVEBUFFER*, WAVEFORMATEX*, char*, unsigned int);
		void				freeBuffer(WAVEBUFFER*);

};

#endif

/* Fin de CResampler.hpp */
