/*==============================================================================
	CSample.cpp
==============================================================================*/

#include "CSample.hpp"

/*==============================================================================
	Constantes
==============================================================================*/

#define BUFFER_SIZE		8*1024

/*==============================================================================
	Constructeur
==============================================================================*/

CSample::CSample(char *lpszFileName) :
	pointerX(0), pointerY(0), nextSample(0),
	_bSelected(0), _nTimePosition(0), _nTrack(0),
	_hFile(0), _fStartPos(0), _fCurrentPos(0), _fEndPos(0),
	_gWaveOutBuffer(0)
{

	strcpy(lpFileName, lpszFileName);
	load();
	close();

}

/*==============================================================================
	Destructeur
==============================================================================*/

CSample::~CSample()
{

	close();
	if ( _gWaveOutBuffer )
		delete _gWaveOutBuffer;

}

/*==============================================================================
	load
==============================================================================*/

BOOL CSample::load(void)
{

	WAVEFILEHEADER		wFileHeader;
	unsigned long		dwBuf;
	int					i;

	// Ferme le fichier s'il y en a un d'ouvert
	close();

	// Ouvre le fichier
	_hFile		= fopen(lpFileName, "rb");

	// Erreur : le fichier n'a pas t ouvert
	if ( !_hFile )
		return FALSE;

	// Vrifie la validit du wav
	fread(&wFileHeader, sizeof(wFileHeader), 1, _hFile);
	if ( wFileHeader.RIFF_const	!= 0x46464952	|| // RIFF
		 wFileHeader.fileSize 	== 0			||
		 wFileHeader.WAVE_const	!= 0x45564157	|| // WAVE
		 wFileHeader._fmt_const	!= 0x20746D66 ) {  // _fmt
		close();
		return FALSE;
	}

	// Lit l'en-tte de format wav
	dwBuf = wFileHeader.formatSize;
	if ( dwBuf == sizeof(PCMWAVEFORMAT) ) { // PCM
		fread(&_wFormatEx, dwBuf, 1, _hFile);
		_wFormatEx.cbSize = 0;
	} else if ( dwBuf == sizeof(WAVEFORMATEX) ) { // Autre forme du PCM
		fread(&_wFormatEx, dwBuf, 1, _hFile);
	} else { // Format non-support
		close();
		return FALSE;
	}

	// Vrifie l'en-tte data
	fread(&dwBuf, sizeof(dwBuf), 1, _hFile);
	if ( dwBuf != 0x61746164 ) { // data
		close();
		return FALSE;
	}

	// Lit la taille des donnes restantes
	fread(&dwBuf, sizeof(dwBuf), 1, _hFile);
	_fEndPos			= _fStartPos + dwBuf;

	// Si le buffer existe, on le supprime
	if ( _gWaveOutBuffer )
		delete _gWaveOutBuffer;

	// Prpare le buffer
	_gWaveOutBuffer		= new CWaveOutBuffer((long)this, stWriteProc,
		_wFormatEx, _wFormatEx.nChannels*2, BUFFER_SIZE);

	return TRUE;

}

/*==============================================================================
	close
==============================================================================*/

void CSample::close(void)
{

	stop();
	if ( _hFile )
		fclose(_hFile);
	_hFile = 0;

}

/*==============================================================================
	play
==============================================================================*/

BOOL CSample::play(void)
{

	// Si le wav est en lecture, on le stoppe
	stop();

	// Si le fichier n'est pas ouvert
	if ( !_hFile && !load() )
		return FALSE;

	// Corrige la position du curseur du fichier
	fseek(_hFile, _fStartPos, SEEK_SET);
	_fCurrentPos		= ftell(_hFile);

	// Commence la mise en cache
	_gWaveOutBuffer->start();

	return TRUE;

}

/*==============================================================================
	pause
==============================================================================*/

/*==============================================================================
	stop
==============================================================================*/

void CSample::stop(void)
{

	// Stoppe le son
	if ( _gWaveOutBuffer )
		_gWaveOutBuffer->stop();

}

/*==============================================================================
	writeProc
==============================================================================*/

int CSample::writeProc(char *lpBuf)
{

	DWORD		dwRestSize;
	int			i, j;

	// Ce qu'on veut lire
	dwRestSize				= BUFFER_SIZE;

	// Si le fichier est tout lu, on stoppe le son
	if ( _fCurrentPos >= _fEndPos )
		return -1;

	// Ce qu'il reste  lire
	j = ( _fEndPos - _fStartPos ) - _fCurrentPos;
    if ( j < dwRestSize ) {
    	i = fread(lpBuf, 1, j, _hFile);
		dwRestSize			-= j;
	} else {
		i = fread(lpBuf, 1, BUFFER_SIZE, _hFile);
		dwRestSize			= 0;
	}

	// Actualise la position
	_fCurrentPos					= ftell(_hFile);
	return i;

}

/*==============================================================================
	stWriteProc
==============================================================================*/

int CSample::stWriteProc(LPVOID param, char *lpBuf)
{

	CSample		*pData;

	pData		= (CSample *)param;
	if ( pData )
		return pData->writeProc(lpBuf);
	return -1;

}

/*==============================================================================
	getFormat
==============================================================================*/

WAVEFORMATEX CSample::getFormat(void)
{

	return _wFormatEx;

}

/*==============================================================================
	getLength
==============================================================================*/

unsigned long CSample::getLength(void)
{

	unsigned long		iWaveSize, iSizePerSec;

	iWaveSize		= _fEndPos - _fStartPos;
	iSizePerSec		= _wFormatEx.nSamplesPerSec * _wFormatEx.nBlockAlign;

	if ( iWaveSize == 0 )
		iWaveSize	= 1;
	if ( iSizePerSec == 0 )
		iSizePerSec	= 1;

	return ( iWaveSize * 1000 / iSizePerSec );

}

/*==============================================================================
	getTimePosition
==============================================================================*/

unsigned long CSample::getTimePosition(void)
{

	return _nTimePosition;

}

/*==============================================================================
	getTrack
==============================================================================*/

unsigned int CSample::getTrack(void)
{

	return _nTrack;

}

/*==============================================================================
	setTimePosition
==============================================================================*/

void CSample::setTimePosition(unsigned long iTime)
{

	_nTimePosition		= iTime;

}

/*==============================================================================
	setTrack
==============================================================================*/

void CSample::setTrack(unsigned int iTrack)
{

	_nTrack				= iTrack;

}

/*==============================================================================
	select
==============================================================================*/

void CSample::select(void)
{

	_bSelected			= TRUE;

}

/*==============================================================================
	unselect
==============================================================================*/

void CSample::unselect(void)
{

	_bSelected			= FALSE;

}

/*==============================================================================
	isSelected
==============================================================================*/

BOOL CSample::isSelected(void)
{

	return _bSelected;

}

/*==============================================================================
	monoToStereo
==============================================================================*/

void CSample::monoToStereo(WAVETRANSFORM *wTf, int iBlockSize)
{

	int			nSrcBlocks, nDestBlocks;
	int			i, j, k;

	nSrcBlocks			= wTf->srcLength / iBlockSize;
	nDestBlocks			= wTf->destLength / iBlockSize;

	if ( nSrcBlocks > nDestBlocks )
		nSrcBlocks		= nDestBlocks;

	for ( i = 0; i < nSrcBlocks; i++ ) {
		for ( j = 0; j < 2; j++ ) {
			for ( k = 0; k < iBlockSize; k++ )
				wTf->destBuf[ 2 * iBlockSize * i + iBlockSize * j + k ]	 = wTf->srcBuf[ iBlockSize * i + k ];
		}
	}

}

/*==============================================================================
	getBuffer
==============================================================================*/

int CSample::getBuffer(char *lpBuf, WAVEFORMATEX *lpWfx, unsigned long iBlockPos, int iBufferSize)
{

	CWaveConverter		*gWaveConverter;
	WAVETRANSFORM		wTransform;
	char				*lpFileBuf;
	unsigned long		iGetTimePosition, iGetLength;
	unsigned long		iFileBlockPos;
	unsigned long		iBufferOffset, iFileOffset;
	float				iRatio;
	int					iFileBufferSize;
	int					iBytesRead;
	int					i, j, k;

	// Vrifie que le fichier est ouvert
	if ( !_hFile && !load() )
		return 0;

	// Rcupre les infos
	iGetTimePosition		= getTimePosition();
	iGetLength				= getLength();

	// Calcule la taille du buffer fichier correspondant
	iRatio					= (float)_wFormatEx.nSamplesPerSec / (float)lpWfx->nSamplesPerSec;
	iFileBufferSize			= (int)( ( iBufferSize / lpWfx->nBlockAlign ) * iRatio ) * _wFormatEx.nBlockAlign;
	lpFileBuf				= new char[ iFileBufferSize ];

	//if ( iRatio != 1 )
	//	return -1;

	// Calcule la position du premier bloc du fichier par rapport  la table
	iFileBlockPos			= lpWfx->nSamplesPerSec * iGetTimePosition / 1000;

	if ( iFileBlockPos >= iBlockPos ) {

		// Si le dbut du sample est aprs le curseur
		iBufferOffset		= ( iFileBlockPos - iBlockPos ) * _wFormatEx.nBlockAlign;
		iFileOffset			= 0;

	} else {

		// S'il est avant
		iBufferOffset		= 0;
		iFileOffset			= ( iBlockPos - iFileBlockPos ) * _wFormatEx.nBlockAlign;

	}

	// Si le curseur est trop avant le sample
	if ( iBufferOffset >= iFileBufferSize )
		return -1;

	// Si le curseur est au-del de la fin du sample
	if ( iFileOffset >= _fEndPos - _fStartPos )
		return -1;

	// Positionne le sample
	fseek(_hFile, _fStartPos + iFileOffset, SEEK_SET);

	// Fait le silence
	ZeroMemory(lpFileBuf, iFileBufferSize);
	ZeroMemory(lpBuf, iBufferSize);

	// Retourne le nombre d'octets lus
	iBytesRead		= fread(lpFileBuf + iBufferOffset, 1, iFileBufferSize - iBufferOffset, _hFile);

	// Fait le resampling du son

	DWORD				dwInputBytesUsed, dwOutputBytesUsed;
	gWaveConverter		= new CWaveConverter();
	gWaveConverter->open(&_wFormatEx, lpWfx);
	gWaveConverter->convert((BYTE*)lpFileBuf, iFileBufferSize, dwInputBytesUsed, (BYTE*)lpBuf, dwOutputBytesUsed);

	//CopyMemory(lpBuf, lpFileBuf, min(iBufferSize, iFileBufferSize - iBufferOffset));
	// Mono -> Stro
	/*if ( lpWfx->nChannels == 2 && _wFormatEx.nChannels == 1 ) {

		// Alloue de la mmoire
		lpTransformBuf	= new char[ iBufferSize ];
		ZeroMemory(lpTransformBuf, iBufferSize);

		wTransform.srcBuf		= lpFileBuf;
		wTransform.srcLength	= iFileBufferSize;
		wTransform.destBuf		= lpTransformBuf;
		wTransform.destLength	= iFileBufferSize * 2;

		monoToStereo(&wTransform, _wFormatEx.wBitsPerSample / 8);

		lpTmpBuf				= lpFileBuf;
		lpFileBuf				= lpTransformBuf;

		delete [] lpTmpBuf;

	}*/

	/*if ( iRatio == 1 ) {

		// Si le nombres de bits/chantillon est gal, on copie le buffer tel quel
		CopyMemory(lpBuf, lpFileBuf, iBufferSize);

	}*/

	delete [] lpFileBuf;

	return 1;

}

/* Fin de CSample.cpp */
