/*==============================================================================
	CSampleCollection.cpp
==============================================================================*/

#include "CSampleCollection.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CSampleCollection::CSampleCollection(CMainDialog* gDialog) :
	gParent(gDialog), dIsDraging(0), gPreview(0)
{

	LVCOLUMN		pCol;
	HIMAGELIST		hImgListNormal,	hImgListSmall;
	HICON			hIconNormal,	hIconSmall;
	POINT			iconSizeNormal,	iconSizeSmall;
	const int		maxImgList = 1;

	// Rcupre le handle du contrle
	hWnd 				= GetDlgItem(gDialog->hDlg, IDC_SAMPLES_LIST);

	// Rcupre la procdure callback
	SetWindowLong(hWnd, GWL_USERDATA, (LONG)this);
	defWndProc			= (WNDPROC)SetWindowLong(hWnd, GWL_WNDPROC, (LONG)stListViewProc);

	// Cre la liste
	sampleCollection	= new CSampleList();

	// Attribue un style
	ListView_SetExtendedListViewStyle(hWnd, LVS_EX_HEADERDRAGDROP|LVS_EX_FULLROWSELECT);

	// Ajoute des colonnes
	pCol.mask 			= LVCF_TEXT|LVCF_WIDTH;
	pCol.pszText 		= "Sample";
	pCol.cx 			= 160;
	ListView_InsertColumn(hWnd, 0, &pCol);

	// Rcupre les dimensions d'une icne grande ou petite
	iconSizeNormal.x	= GetSystemMetrics(SM_CXICON);
	iconSizeNormal.y	= GetSystemMetrics(SM_CYICON);
	iconSizeSmall.x		= GetSystemMetrics(SM_CXSMICON);
	iconSizeSmall.y		= GetSystemMetrics(SM_CYSMICON);

	// Cre une liste d'icnes
	hImgListNormal		= ImageList_Create(iconSizeNormal.x, iconSizeNormal.y,
								ILC_COLOR16|ILC_MASK, maxImgList, 0);
	hImgListSmall		= ImageList_Create(iconSizeSmall.x, iconSizeSmall.y,
								ILC_COLOR16|ILC_MASK, maxImgList, 0);

	// Charge les icnes en mmoire
	hIconNormal			= (HICON)LoadImage(gParent->getInstance(), MAKEINTRESOURCE(IDI_SOUND),
								IMAGE_ICON, iconSizeNormal.x, iconSizeNormal.y, LR_CREATEDIBSECTION);
	hIconSmall			= (HICON)LoadImage(gParent->getInstance(), MAKEINTRESOURCE(IDI_SOUND),
								IMAGE_ICON, iconSizeSmall.x, iconSizeSmall.y, LR_CREATEDIBSECTION);

	// Paramtres de la liste d'images
	ImageList_AddIcon(hImgListNormal, hIconNormal);
	ImageList_AddIcon(hImgListSmall,  hIconSmall);
	ImageList_SetBkColor(hImgListNormal, CLR_NONE);
	ImageList_SetBkColor(hImgListSmall,  CLR_NONE);

	// Ajoute la liste d'icnes  la listview
	ListView_SetImageList(hWnd, hImgListNormal, LVSIL_NORMAL);
	ListView_SetImageList(hWnd, hImgListSmall, LVSIL_SMALL);

	// Libre la mmoire
	DestroyIcon(hIconNormal);
	DestroyIcon(hIconSmall);

}

/*==============================================================================
	Destructeur
==============================================================================*/

CSampleCollection::~CSampleCollection()
{

	deleteAllFiles();
	delete sampleCollection;

}

/*==============================================================================
	showAddSampleDialog
==============================================================================*/

void CSampleCollection::showAddSampleDialog(void)
{

	OPENFILENAME 		ofn;
	char				lpszFileName[MAX_PATH*2];
	char				lpszThisFile[MAX_PATH];
	int					ptOffset, len;

	ZeroMemory(lpszFileName, MAX_PATH*2);
	ZeroMemory(&ofn, sizeof(ofn));

	ofn.lStructSize		= sizeof(ofn);
	ofn.hwndOwner		= gParent->hDlg;
	ofn.lpstrTitle		= "Ajouter un sample";
	ofn.lpstrFilter		= "Fichier WAVE (*.wav)\0*.wav\0Tous les fichiers (*.*)\0*.*\0\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= lpszFileName;
	ofn.nMaxFile		= MAX_PATH;
	ofn.Flags			= OFN_ALLOWMULTISELECT|OFN_NOCHANGEDIR|OFN_EXPLORER|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;

	if ( GetOpenFileName(&ofn) ) {
		ptOffset		= ofn.nFileOffset;
		if ( ptOffset < strlen(lpszFileName) ) { // Un fichier
			addFile(lpszFileName);
		} else { // Plusieurs fichiers
			CopyMemory(lpszThisFile, lpszFileName, ofn.nFileOffset);
			lpszThisFile[ofn.nFileOffset - 1] = '\\';
			while ( lpszFileName[ptOffset] != '\0' ) {
				len				= strlen(lpszFileName + ptOffset) + 1;
				CopyMemory(lpszThisFile + ofn.nFileOffset, lpszFileName + ptOffset, len);
				addFile(lpszThisFile);
				ptOffset		+= len;
			}
		}
	}

}

/*==============================================================================
	showSamplePropDialog
==============================================================================*/

void CSampleCollection::showSamplePropDialog(void)
{

	CSamplePropDialog *SamplePropWindow;
	CSample			*pSample;
	LVITEM			pItem;
	int				i;

	// Trouve le premier sample slectionn
	i = ListView_GetNextItem(hWnd, (WPARAM)-1, LVNI_SELECTED);

	if ( i >= 0 ) {

		// Rcupre le pointeur du sample
		ZeroMemory(&pItem, sizeof(pItem));
		pItem.mask			= LVIF_PARAM;
		pItem.iItem			= i;
		ListView_GetItem(hWnd, &pItem);
		pSample				= (CSample *)pItem.lParam;

		// Ouvre la boite de dialogue
		SamplePropWindow = new CSamplePropDialog(gParent->getInstance(), gParent->hDlg, pSample);
		delete SamplePropWindow;

	}

}

/*==============================================================================
	addFile
==============================================================================*/

void CSampleCollection::addFile(char *lpszFileName)
{

	CSample			*pSample;
	LVITEM			pItem;
	int				bInt, index;
	int				i = 0, j = 0;

	if ( !lpszFileName )
		return;
	if ( *lpszFileName == 0 )
		return;

	// Valide le wav
	pSample				= new CSample(lpszFileName);
	if ( !pSample->load() ) {
		delete pSample;
		MessageBox(gParent->hDlg, "Le fichier n'est pas un son wave valide.", "Erreur", MB_ICONERROR);
		return;
	}

	// Le place dans la liste virtuelle
	sampleCollection->add(pSample);

	// Rcupre le nom de fichier
	while ( pSample->lpFileName[i++] != 0 ) {
		if ( pSample->lpFileName[i] == '\\' )
			j = i + 1;
	}

	// Ajoute un lment
	ZeroMemory(&pItem, sizeof(pItem));
	pItem.mask 			= LVIF_TEXT|LVIF_IMAGE|LVIF_PARAM;
	pItem.iItem			= ListView_GetItemCount(hWnd);
	pItem.pszText		= pSample->lpFileName + j;
	pItem.iImage		= 0;
	pItem.lParam		= (LPARAM)pSample->lpFileName;
	index				= ListView_InsertItem(hWnd, &pItem);

	// Affiche le texte
	ListView_SetItemText(hWnd, index, 0, pSample->lpFileName + j);

}

/*==============================================================================
	actualize
==============================================================================*/

void CSampleCollection::actualize(void)
{

	CSample			*pSample;
	LVITEM			pItem;
	int				i, j = 0, k = 0;

	// Supprime tous les lments rels
	ListView_DeleteAllItems(hWnd);

	for ( i = 0; i < sampleCollection->count(); i++ ) {

		// Rcupre l'lment courant
		pSample				= sampleCollection->getByIndex(i);

		// Rcupre le nom de fichier
		while ( pSample->lpFileName[j++] != 0 ) {
			if ( pSample->lpFileName[j] == '\\' )
				k = j + 1;
		}

		// Cre un nouvel lment rel
		ZeroMemory(&pItem, sizeof(pItem));
		pItem.mask 			= LVIF_TEXT|LVIF_IMAGE|LVIF_PARAM;
		pItem.iItem			= ListView_GetItemCount(hWnd);
		pItem.pszText		= pSample->lpFileName + k;
		pItem.iImage		= 0;
		pItem.lParam		= (LPARAM)pSample;
		i = ListView_InsertItem(hWnd, &pItem);

		// Elements
		ListView_SetItemText(hWnd, i, 0, pSample->lpFileName + k);

	}

}

/*==============================================================================
	deleteFile
==============================================================================*/

void CSampleCollection::deleteFile(CSample *pSample)
{

	sampleCollection->remove(pSample);

}

/*==============================================================================
	deleteSelectedFiles
==============================================================================*/

void CSampleCollection::deleteSelectedFiles(void)
{

	CSample			*pSample;
	LVITEM			pItem;
	int				i;

	do {
		i = ListView_GetNextItem(hWnd, (WPARAM)-1, LVNI_SELECTED);
		if ( i >= 0 ) {
			ZeroMemory(&pItem, sizeof(pItem));
			pItem.mask			= LVIF_PARAM;
			pItem.iItem			= i;
			ListView_GetItem(hWnd, &pItem);
			pSample				= (CSample *)pItem.lParam;
			deleteFile(pSample);
			ListView_DeleteItem(hWnd, i);
		}
	} while ( i >= 0 );

}

/*==============================================================================
	deleteAllFiles
==============================================================================*/

void CSampleCollection::deleteAllFiles(void)
{

	sampleCollection->clear();
	ListView_DeleteAllItems(hWnd);

}

/*==============================================================================
	onNotify
==============================================================================*/

void CSampleCollection::onNotify(HWND hDlg, LPARAM lParam)
{

	NMHDR			*nmhdr;

	nmhdr			= (NMHDR *)lParam;

	switch ( nmhdr->code ) {

		case LVN_BEGINDRAG:
			beginDrag(hDlg, (NMLISTVIEW *)lParam);
			break;

		case LVN_ENDLABELEDIT:
			onEndLabelEdit((NMLVDISPINFO *)lParam);
			break;

		case LVN_KEYDOWN:
			onKeyDown((NMLVKEYDOWN *)lParam);
			break;

		case NM_DBLCLK:
			onLeftButtonDoubleClick((NMITEMACTIVATE *)lParam);
			break;

		case NM_RCLICK:
			onRightButtonDown((NMITEMACTIVATE *)lParam);
			break;

	}

}

/*==============================================================================
	beginDrag
==============================================================================*/

void CSampleCollection::beginDrag(HWND hDlg, NMLISTVIEW *nmlv)
{

	LVITEM			pItem;
	POINT			pt;
	int				srcItem;

	// Rcupre l'lment
	srcItem				= nmlv->iItem;

	// Rcupre le lParam de l'lment
	ZeroMemory(&pItem, sizeof(pItem));
	pItem.mask			= LVIF_PARAM;
	pItem.iItem			= srcItem;
	ListView_GetItem(hWnd, &pItem);
	dItemDragParam		= pItem.lParam;

	// Cre une image de l'lment slectionn
	dDragImageList		= (HIMAGELIST)ListView_CreateDragImage(hWnd, srcItem, &pt);

	// Fait flotter l'image et restreint le drag and drop  l'application seulement
	if ( ImageList_BeginDrag(dDragImageList, 0, 0, 0) ) {
		ImageList_DragShowNolock(TRUE);
		SetCursor(LoadCursor(NULL, IDC_HAND));
		SetCapture(hDlg);
		dIsDraging		= TRUE;
	}

}

/*==============================================================================
	moveDrag
==============================================================================*/

void CSampleCollection::moveDrag(HWND hDlg, LPARAM lParam)
{

	POINT			iPoint;

	// Fait flotter l'lment si Drag and Drop il y a
	if ( dIsDraging ) {
		iPoint.x		= (short)LOWORD(lParam) + 16;
		iPoint.y		= (short)HIWORD(lParam) + 16;
		ClientToScreen(gParent->hDlg, &iPoint);
		ImageList_DragMove(iPoint.x, iPoint.y);
	}

}

/*==============================================================================
	endDrag
==============================================================================*/

void CSampleCollection::endDrag(HWND hDlg, LPARAM lParam)
{

	HWND			dhDropCible;
	LVITEM			pItem;
	POINT			iPoint;
	RECT			rc;
	int				x, y;

	if ( !dIsDraging )
		return;

	// Dbloque le Drag and Drop
	ReleaseCapture();
	ImageList_EndDrag();
	ImageList_Destroy(dDragImageList);
	dIsDraging			= FALSE;

	// Rcupre la Handle du contrle sous le curseur
	iPoint.x			= LOWORD(lParam);
	iPoint.y			= HIWORD(lParam);
	dhDropCible			= ChildWindowFromPoint(hDlg, iPoint);

	if ( dhDropCible ) {

		// Rcupre les coordonnes relatives de la cible
		GetWindowRect(dhDropCible, &rc);
		iPoint.x			= rc.left;
		iPoint.y			= rc.top;
		ScreenToClient(gParent->hDlg, &iPoint);

		// Rcupre les coordonnes relatives du curseur
		x = LOWORD(lParam) - iPoint.x;
		y = HIWORD(lParam) - iPoint.y;

		// Envoie un message  la cible
		PostMessage(dhDropCible, WM_USER+1, MAKEWPARAM(x, y), (LPARAM)dItemDragParam);

	}

}

/*==============================================================================
	onEndLabelEdit
==============================================================================*/

void CSampleCollection::onEndLabelEdit(NMLVDISPINFO *nmdi)
{

	LVITEM			pItem;
	HWND			hWndEdit;
	char			*lpszText;
	int				len;

	// Rcupre la structure de l'lment modifi
	pItem			= nmdi->item;

	// Copie le nouveau nom du sample
	hWndEdit		= ListView_GetEditControl(hWnd);
	len				= SendMessage(hWndEdit, WM_GETTEXTLENGTH, 0, 0) + 1;
	lpszText		= new char[len+1];
	SendMessage(hWndEdit, WM_GETTEXT, len, (LPARAM)lpszText);

	// Modifie le nom du sample
	ListView_SetItemText(hWnd, nmdi->item.iItem, 0, lpszText);

	// Libre la mmoire
	delete [] lpszText;

}

/*==============================================================================
	onKeyDown
==============================================================================*/

void CSampleCollection::onKeyDown(NMLVKEYDOWN *nmkd)
{

	if ( nmkd->wVKey == VK_DELETE )
		deleteSelectedFiles();

}

/*==============================================================================
	onLeftButtonDoubleClick
==============================================================================*/

void CSampleCollection::onLeftButtonDoubleClick(NMITEMACTIVATE *nmia)
{

	LVITEM			pItem;
	int				srcItem;

	// Rcupre l'lment
	srcItem				= nmia->iItem;

	if ( srcItem < 0 )
		return;

	// Si un son est dj jou, on l'arrte
	if ( gPreview )
		gPreview->stop();

	// Rcupre le lParam de l'lment
	ZeroMemory(&pItem, sizeof(pItem));
	pItem.mask			= LVIF_PARAM;
	pItem.iItem			= srcItem;
	ListView_GetItem(hWnd, &pItem);
	gPreview			= (CSample *)pItem.lParam;

	// Cre un nouveau son et le joue
	gPreview->play();

}

/*==============================================================================
	onRightButtonDown
==============================================================================*/

void CSampleCollection::onRightButtonDown(NMITEMACTIVATE *nmia)
{

	HINSTANCE		hInstance;
	HMENU			hMenu, hPopup;
	POINT			iPoint;

	// Rcupre l'instance
	hInstance		= (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE);

	// Convertit les coordonnes
	iPoint			= nmia->ptAction;
	ClientToScreen(hWnd, &iPoint);

	if ( nmia->iItem >= 0 )
		hMenu		= LoadMenu(hInstance, MAKEINTRESOURCE(IDR_SAMPLECOLLECTION_ITEM));
	else
		hMenu		= LoadMenu(hInstance, MAKEINTRESOURCE(IDR_SAMPLECOLLECTION));

	// Rcupre le sous-menu
	hPopup			= GetSubMenu(hMenu, 0);

	// Affiche le menu
	TrackPopupMenuEx(hPopup, 0, iPoint.x, iPoint.y, hWnd, NULL);
	DestroyMenu(hMenu);

}

/*==============================================================================
	onCommand
==============================================================================*/

void CSampleCollection::onCommand(WPARAM wParam, LPARAM lParam)
{

	switch ( LOWORD(wParam) ) {

		case IDR_SAMPLECOLLECTION_ADD:
			showAddSampleDialog();
			break;

		case IDM_SAMPLECOLLECTION_DELETEALL:
			deleteAllFiles();
			break;

		case IDM_SAMPLECOLLECTION_ITEM_PROPERTIES:
			showSamplePropDialog();
			break;

		case IDM_SAMPLECOLLECTION_ITEM_DELETE:
			deleteSelectedFiles();
			break;

	}

}

/*==============================================================================
	onDropFiles
==============================================================================*/

void CSampleCollection::onDropFiles(WPARAM wParam)
{

	HDROP			hDrop;
	char			szFile[MAX_PATH];
	int				iCountFiles;
	int				i;

	// Rcupre le handle DropFiles
	hDrop			= (HDROP)wParam;

	// Rcupre le nombre de fichiers
	iCountFiles		= DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);

	for ( i = 0; i < iCountFiles; i++ ) {

		// Rcupre le nom de fichier dpos et l'ajoute
		DragQueryFile(hDrop, i, szFile, sizeof(szFile));
		addFile(szFile);

	}

	// Libre la mmoire utilise
	DragFinish(hDrop);

}

/*==============================================================================
	stListViewProc
==============================================================================*/

LRESULT CALLBACK CSampleCollection::stListViewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	CSampleCollection	*pData;

	pData 			= (CSampleCollection *)GetWindowLong(hWnd, GWL_USERDATA);

	if ( pData ) {

		switch ( uMsg ) {

			case WM_COMMAND:
				pData->onCommand(wParam, lParam);
				return 0;

			case WM_DROPFILES:
				pData->onDropFiles(wParam);
				return 0;

			default:
				return CallWindowProc(pData->defWndProc, hWnd, uMsg, wParam, lParam);

		}

	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);

}

/* Fin de CSampleCollection.cpp */
