/*==============================================================================
	CSampleList.cpp
==============================================================================*/

#include "CSampleList.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CSampleList::CSampleList() : pStartSample(0)
{
}

/*==============================================================================
	Destructeur
==============================================================================*/

CSampleList::~CSampleList()
{

	clear(1);

}

/*==============================================================================
	add
==============================================================================*/

CSample *CSampleList::add(CSample *gSample)
{

	CSample				*pCurrentSample;
	CSample				*pPrevSample;

	pPrevSample					= 0;
	pCurrentSample				= pStartSample;

	while ( pCurrentSample != 0 ) {
		if ( pCurrentSample == gSample )
			return pCurrentSample;
		pPrevSample				= pCurrentSample;
		pCurrentSample			= pCurrentSample->nextSample;
	}

	if ( pPrevSample )
		pPrevSample->nextSample		= gSample;
	else
		pStartSample				= gSample;
	gSample->nextSample	= 0;
	return gSample;

}

/*==============================================================================
	remove
==============================================================================*/

CSample *CSampleList::remove(CSample *gSample, int bDelete)
{

	CSample				*pCurrentSample;
	CSample				*pPrevSample;

	pPrevSample				= 0;
	pCurrentSample			= pStartSample;

	while ( pCurrentSample != 0 ) {
		if ( pCurrentSample == gSample ) {
			if ( pPrevSample )
				pPrevSample->nextSample		= pCurrentSample->nextSample;
			else
				pStartSample				= pCurrentSample->nextSample;
			if ( bDelete ) {
				delete gSample;
				return 0;
			} else {
				return gSample;
			}
		}
		pPrevSample			= pCurrentSample;
		pCurrentSample		= pCurrentSample->nextSample;
	}
	return 0;

}

/*==============================================================================
	clear
==============================================================================*/

void CSampleList::clear(int bDelete)
{

	CSample				*pCurrentSample;
	CSample				*pNextSample;

	pCurrentSample			= pStartSample;

	while ( pCurrentSample != 0 ) {
		pNextSample			= pCurrentSample->nextSample;
		if ( bDelete )
			delete pCurrentSample;
		else
			pCurrentSample->nextSample	= 0;
		pCurrentSample		= pNextSample;
	}

	pStartSample			= 0;

}

/*==============================================================================
	count
==============================================================================*/

int CSampleList::count(void)
{

	CSample				*pCurrentSample;
	int					i = 0;

	pCurrentSample			= pStartSample;

	while ( pCurrentSample != 0 ) {
		i++;
		pCurrentSample		= pCurrentSample->nextSample;
	}
	return i;

}

/*==============================================================================
	empty
==============================================================================*/

int CSampleList::empty(void)
{

	return ( pStartSample == 0 );

}

/*==============================================================================
	getByIndex
==============================================================================*/

CSample *CSampleList::getByIndex(int index)
{

	CSample				*pCurrentSample;
	int					i = 0;

	pCurrentSample			= pStartSample;

	while ( pCurrentSample != 0 ) {
		if ( i == index )
			return pCurrentSample;
		pCurrentSample		= pCurrentSample->nextSample;
		i++;
	}
	return 0;

}

/*==============================================================================
	swap
==============================================================================*/

void CSampleList::swap(int a, int b)
{

	CSample				*pCurrentSample, *pPrevSample;
	CSample				*pSamplePrevA, *pSampleA, *pSampleB, *pSampleNextB;
	int					c, i;

	// Nombre d'lments
	c						= count();

	// Si les index sont invalides
	if ( a < 0 || b < 0 || a >= c || b >= c || a == b )
		return;

	// a doit tre plus petit que b
	if ( a > b ) {
		i = a;
		a = b;
		b = i;
	}

	i						= 0;
	pSampleA				= 0;
	pSampleB				= 0;
	pSamplePrevA			= 0;
	pSampleNextB			= 0;
	pPrevSample				= 0;
	pCurrentSample			= pStartSample;

	while ( pCurrentSample != 0 ) {
		if ( i == a ) {
			pSampleA		= pCurrentSample;
			pSamplePrevA	= pPrevSample;
		}
		if ( i == b )
			break;
		pPrevSample			= pCurrentSample;
		pCurrentSample		= pCurrentSample->nextSample;
		i++;
	}

	// C'est le sample courant
	pSampleB		= pCurrentSample;

	// Modifie les samples prcdent
	if ( pSamplePrevA )
		pSamplePrevA->nextSample	= pSampleB;
	else
		pStartSample				= pSampleB;
	pPrevSample->nextSample			= pSampleA;

	// Modifie les pointeurs vers les samples suivants
	pSampleNextB					= pSampleB->nextSample;
	pSampleB->nextSample			= pSampleA->nextSample;
	pSampleA->nextSample			= pSampleNextB;

}

/*==============================================================================
	sortByTimePosition
==============================================================================*/

void CSampleList::sortByTimePosition(void)
{

	CSample			**pSampleArray, *pCurrentSample;
	unsigned long	iMinTime;
	int				c, i, j, index;

	// Compte le nombre d'lments
	c						= count();

	// Cre un tableau de pointeurs
	pSampleArray			= new CSample*[c];

	// Copie les pointeurs de la liste dans le tableau temporaire
	pCurrentSample			= pStartSample;
	i						= 0;
	while ( pCurrentSample != 0 ) {
		pSampleArray[i++]	= pCurrentSample;
		pCurrentSample		= pCurrentSample->nextSample;
	}

	// Efface la liste sans effacer les samples
	clear(0);

	for ( i = 0; i < c; i++ ) {

		// Le temps ne peut excder le maximum d'un unsigned long
		iMinTime			= 0xFFFFFFFF;
		index				= -1;

		// Enumre les lments et ne retient que la plus petite position
		for ( j = 0; j < c; j++ ) {
			if ( pSampleArray[j] != 0 && pSampleArray[j]->getTimePosition() <= iMinTime ) {
				index		= j;
				iMinTime	= pSampleArray[j]->getTimePosition();
			}
		}

		// Transfre l'lment courant ayant la plus petite position dans la liste
		if ( index >= 0 && pSampleArray[index] != 0 ) {
			add(pSampleArray[index]);
			pSampleArray[index]		= 0;
		}
	}

	// Libre la mmoire
	delete [] pSampleArray;

}

/* Fin de CSampleList.cpp */
