/*==============================================================================
	CSplitter.cpp
==============================================================================*/

#include "CSplitter.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CSplitter::CSplitter(HWND hDlg, WORD id, HWND h1, HWND h2, int defPos, DWORD vert)
		: _hParent(hDlg), _hPaneFirst(h1), _hPaneSecond(h2), _bIsVert(vert), _bIsMoving(0)
{

	WNDCLASSEX		wc;
	HINSTANCE		hInstance;

	// Proprits
	dwStyle 			= WS_VISIBLE|WS_CHILD;
	dwStyleEx 			= 0;
	wId					= id;

	// Rglages
	_iMinFirst			= 5;
	_iMinSecond			= 5;
	_iBorderSize		= 3;

	// Rcupre l'instance
	hInstance			= (HINSTANCE)GetWindowLong(hDlg, GWL_HINSTANCE);

	// Construit une nouvelle classe
	ZeroMemory(&wc, sizeof(wc));
	wc.cbSize			= sizeof(wc);
	wc.style			= CS_DBLCLKS|CS_HREDRAW|CS_VREDRAW;
	wc.hInstance		= hInstance;
	wc.lpszClassName	= "SWM_CSplitter_WndClass";

	// Curseur
	if ( !_bIsVert )
		wc.hCursor			= LoadCursor(NULL, IDC_SIZENS);
	else
		wc.hCursor			= LoadCursor(NULL, IDC_SIZEWE);

	// Enregistre la classe
	registerClass(&wc);

	// Ouvre le gadget
	open(hInstance, hDlg);

	// Empche la rinitialisation de la barre du milieu
	if ( !_bIsVert )
		MoveWindow(hWnd, 0, 0, 0, defPos + _iBorderSize * 2, FALSE);
	else
		MoveWindow(hWnd, 0, 0, defPos + _iBorderSize * 2, 0, FALSE);
	_iCurrentPos			= defPos;

}

/*==============================================================================
	setMinimumSizes
==============================================================================*/

void CSplitter::setMinimumSizes(int iMin1, int iMin2)
{

	_iMinFirst		= iMin1;
	_iMinSecond		= iMin2;

}

/*==============================================================================
	rePaint
==============================================================================*/

void CSplitter::rePaint(void)
{


	PAINTSTRUCT		ps;
	HDC				tmpHdc;
	HBRUSH			hEmptyBrush, hSplitterBrush;
	RECT			rcClient, rcBorder;

	// Rcupre les dimensions
	GetClientRect(hWnd, &rcClient);

	// Commence le dessin
	tmpHdc = BeginPaint(hWnd, &ps);

	// cration des objets
	hEmptyBrush			= CreateSolidBrush(GetSysColor(COLOR_3DFACE));
	hSplitterBrush		= CreateSolidBrush(GetSysColor(COLOR_BTNFACE));

	if ( !_bIsVert ) {
		rcBorder.top		= _iCurrentPos - _iBorderSize;
		rcBorder.bottom		= _iCurrentPos + _iBorderSize;
		rcBorder.left		= rcClient.left;
		rcBorder.right		= rcClient.right;
	} else {
		rcBorder.top		= rcClient.top;
		rcBorder.bottom		= rcClient.bottom;
		rcBorder.left		= _iCurrentPos - _iBorderSize;
		rcBorder.right		= _iCurrentPos + _iBorderSize;
	}
	FillRect(tmpHdc, &rcBorder, hEmptyBrush);

	// Fin du dessin
	DeleteObject(hEmptyBrush);
	DeleteObject(hSplitterBrush);
	EndPaint(hWnd, &ps);

}

/*==============================================================================
	onLeftButtonDown
==============================================================================*/

void CSplitter::onLeftButtonDown(void)
{

	// Active le dplacement
	_bIsMoving		= TRUE;

	// Bloque l'accs aux autres fentres
	SetCapture(hWnd);

}

/*==============================================================================
	onMouseMove
==============================================================================*/

void CSplitter::onMouseMove(WPARAM wParam, LPARAM lParam)
{

	short xPos, yPos;

	// Dplace les lments si le bouton gauche de la souris est cliqu
	if ( _bIsMoving != 0 && wParam & MK_LBUTTON ) {

		// Rcupre les coordonnes du curseur
		xPos = (short)LOWORD(lParam);
		yPos = (short)HIWORD(lParam);

		if ( !_bIsVert )
			_iCurrentPos = yPos;
		else
			_iCurrentPos = xPos;
		reSize();

	}

}

/*==============================================================================
	onLeftButtonUp
==============================================================================*/

void CSplitter::onLeftButtonUp(LPARAM lParam)
{

	// Dsactive le dplacement
	_bIsMoving		= FALSE;

	// Dbloque l'accs aux autres contrles
	ReleaseCapture();

}

/*==============================================================================
	onSize
==============================================================================*/

void CSplitter::onSize(WPARAM wParam, LPARAM lParam)
{

	reSize();

}

/*==============================================================================
	reSize
==============================================================================*/

void CSplitter::reSize(void)
{

	RECT		rcClient;

	// Si la fentre n'est pas visible on passe
	if ( IsIconic(_hParent) )
		return;

	// Rcupre les coordonnes et dimensions du client
	getRelativeRect(&rcClient);

	// Rectifie la position si ncssaire
	if ( _iCurrentPos < _iBorderSize + _iMinFirst )
		_iCurrentPos = _iBorderSize + _iMinFirst;

	// Redimensionne les lements
	if ( !_bIsVert ) {
		if ( _iCurrentPos > rcClient.bottom - _iBorderSize - _iMinSecond )
			_iCurrentPos = rcClient.bottom - _iBorderSize - _iMinSecond;
		MoveWindow(_hPaneFirst,
			rcClient.left,
			rcClient.top,
			rcClient.right,
			_iCurrentPos - _iBorderSize, TRUE);
		MoveWindow(_hPaneSecond,
			rcClient.left,
			rcClient.top + _iCurrentPos + _iBorderSize,
			rcClient.right,
			rcClient.bottom - _iCurrentPos - _iBorderSize, TRUE);
	} else {
		if ( _iCurrentPos > rcClient.right - _iBorderSize - _iMinSecond )
			_iCurrentPos = rcClient.right - _iBorderSize - _iMinSecond;
		MoveWindow(_hPaneFirst,
			rcClient.left,
			rcClient.top,
			_iCurrentPos - _iBorderSize,
			rcClient.bottom, TRUE);
		MoveWindow(_hPaneSecond,
			rcClient.left + _iCurrentPos + _iBorderSize,
			rcClient.top,
			rcClient.right - _iCurrentPos - _iBorderSize,
			rcClient.bottom, TRUE);
	}
	rePaint();

}

/*==============================================================================
	getRelativeRect
==============================================================================*/

void CSplitter::getRelativeRect(RECT *rcClient)
{

	POINT		pClientPos;

	// Rcupre les coordonnes absolues (par rapport au bureau)
	GetWindowRect(hWnd, rcClient);

	// Convertit les coordonnes par rapport  la fentre parent
	pClientPos.x		= rcClient->left;
	pClientPos.y		= rcClient->top;
	ScreenToClient(_hParent, &pClientPos);

	// Affecte rcClient
	rcClient->bottom	-= rcClient->top;
	rcClient->right		-= rcClient->left;
	rcClient->left		= pClientPos.x;
	rcClient->top		= pClientPos.y;

}

/*==============================================================================
	gadgetProc
==============================================================================*/

int CSplitter::gadgetProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch ( uMsg ) {

		case WM_PAINT:
			rePaint();
			break;

		case WM_LBUTTONDOWN:
			onLeftButtonDown();
			break;

		case WM_MOUSEMOVE:
			onMouseMove(wParam, lParam);
			break;

		case WM_LBUTTONUP:
            onLeftButtonUp(lParam);
            break;

		case WM_SIZE:
            onSize(wParam, lParam);
            break;

		case WM_DESTROY:
			unregisterClass();
			break;

		default:
			return 0;

	}
	return 1;

}

/* Fin de CSplitter.cpp */
