/*==============================================================================
	CThread.cpp
==============================================================================*/

#include "CThread.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CThread::CThread(LPTHREAD_START_ROUTINE lpThreadProc) : hThread(NULL), dwThreadId(0),
	stThreadProc(lpThreadProc)
{
}

/*==============================================================================
	Destructeur
==============================================================================*/

CThread::~CThread()
{

	terminate();

}

/*==============================================================================
	start
==============================================================================*/

void CThread::start(LPVOID lpArg)
{

	if ( !hThread && stThreadProc )
		hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)stThreadProc, (LPVOID)lpArg, 0, &dwThreadId);

}

/*==============================================================================
	stop
==============================================================================*/

void CThread::stop(DWORD dwTimeOut)
{

	if ( hThread ) {
		//PostThreadMessage(dwThreadId, WM_STOPTHREAD, 0, 0);
		wait(dwTimeOut);
		CloseHandle(hThread);
	}

	hThread			= NULL;
	dwThreadId		= 0;

}

/*==============================================================================
	wait
==============================================================================*/

DWORD CThread::wait(DWORD dwTimeOut)
{

	return WaitForSingleObject(hThread, dwTimeOut);

}

/*==============================================================================
	terminate
==============================================================================*/

void CThread::terminate(void)
{

	if ( hThread ) {
		TerminateThread(hThread, 0);
		CloseHandle(hThread);
	}

	hThread		= NULL;
	dwThreadId	= 0;

}

/*==============================================================================
	setPriority
==============================================================================*/

void CThread::setPriority(int iPriority)
{

	if ( hThread )
		SetThreadPriority(hThread, iPriority);

}

/*==============================================================================
	isRunning
==============================================================================*/

BOOL CThread::isRunning(void)
{

	return ( hThread != NULL );

}

/*==============================================================================
	getHandle
==============================================================================*/

HANDLE CThread::getHandle(void)
{

	return hThread;

}

/*==============================================================================
	getID
==============================================================================*/

DWORD CThread::getID(void)
{

	return dwThreadId;

}

/* Fin de CThread.cpp */
