/*==============================================================================
	CWaveConverter.cpp
==============================================================================*/

#include "CWaveConverter.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CWaveConverter::CWaveConverter()
{

	m_hAcmStream				= NULL;
	memset(&m_AcmStreamHeader, 0, sizeof(m_AcmStreamHeader));
	m_AcmStreamHeader.cbStruct	= sizeof(m_AcmStreamHeader);
	init();

}

CWaveConverter::~CWaveConverter()
{

     init();

}

void CWaveConverter::init(void)
{
     //Close the stream if opened
     if(m_hAcmStream)
     {
          if(m_AcmStreamHeader.fdwStatus & ACMSTREAMHEADER_STATUSF_PREPARED)
          {
               //Cleanup the preparation performed by acmStreamPrepareHeader
            m_AcmStreamHeader.cbSrcLength = m_dwSrcBufferSize;
               m_AcmStreamHeader.cbDstLength = m_dwDstBufferSize;
            acmStreamUnprepareHeader(m_hAcmStream, &m_AcmStreamHeader, 0L);
          }
          acmStreamClose(m_hAcmStream, 0);
          m_hAcmStream = NULL;
     }

     if(m_AcmStreamHeader.pbSrc)
     {
          delete []m_AcmStreamHeader.pbSrc;
     }

     if(m_AcmStreamHeader.pbDst)
     {
          delete []m_AcmStreamHeader.pbDst;
     }
     //Init to zero
     memset(&m_AcmStreamHeader, 0, sizeof(m_AcmStreamHeader));
     m_AcmStreamHeader.cbStruct = sizeof(m_AcmStreamHeader);

     m_dwSrcBufferSize = 0;
     m_dwDstBufferSize = 0;
     m_bStreamOpened = false;
}

BOOL CWaveConverter::open(WAVEFORMATEX *pWfxInput, WAVEFORMATEX *pWfxOutput)
{
     long nGeneralEx(0);

     if(m_bStreamOpened)
     {
          return false;
     }

     try
     {
          if( 0 != acmStreamOpen(&m_hAcmStream,
                              NULL,
                              pWfxInput,
                              pWfxOutput,
                              NULL,
                              0L,
                              0L,
                              ACM_STREAMOPENF_NONREALTIME))
          {
               throw nGeneralEx;
          }

          //Compute source bytes to read (round down to nearest block for
          //one second of data)
          DWORD nAvgBytesPerSec = pWfxInput->nAvgBytesPerSec;
          WORD nBlockAlign = pWfxInput->nBlockAlign;
          m_dwSrcBufferSize = nAvgBytesPerSec - (nAvgBytesPerSec % nBlockAlign);

          if(acmStreamSize(m_hAcmStream,
                              m_dwSrcBufferSize,
                              &m_dwDstBufferSize,
                              ACM_STREAMSIZEF_SOURCE) !=0)
          {
               throw nGeneralEx;
          }

          //Initialize the stream header
         m_AcmStreamHeader.fdwStatus = 0L;
          m_AcmStreamHeader.dwUser = 0L;
          m_AcmStreamHeader.pbSrc = new BYTE[m_dwSrcBufferSize];
          m_AcmStreamHeader.cbSrcLength = m_dwSrcBufferSize;
          m_AcmStreamHeader.cbSrcLengthUsed = 0L;
          m_AcmStreamHeader.dwSrcUser = m_dwSrcBufferSize;
          m_AcmStreamHeader.pbDst = new BYTE[m_dwDstBufferSize];
          m_AcmStreamHeader.cbDstLength = m_dwDstBufferSize;
          m_AcmStreamHeader.cbDstLengthUsed = 0L;
          m_AcmStreamHeader.dwDstUser = m_dwDstBufferSize;

          if(acmStreamPrepareHeader(m_hAcmStream, &m_AcmStreamHeader, 0L) != 0)
          {
               throw nGeneralEx;
          }
          m_bStreamOpened = true;
     }
     catch(long)
     {
          init();
          return false;
     }

     return true;
}

void CWaveConverter::close()
{
     init();
}

DWORD CWaveConverter::getInputBufferSize()
{
     return m_dwSrcBufferSize;
}

DWORD CWaveConverter::getOutputBufferSize()
{
     return m_dwDstBufferSize;
}


BOOL CWaveConverter::convert(const LPBYTE pbInputData, DWORD nInputSize, DWORD &dwInputBytesUsed, LPBYTE pbOutputData, DWORD &dwOutputBytesUsed)
{
     if(!m_bStreamOpened)
     {
          return false;
     }

     //Copy the passed input data to the source buffer of the stream header
     memcpy(m_AcmStreamHeader.pbSrc, pbInputData, nInputSize);
    m_AcmStreamHeader.cbSrcLength = nInputSize;
     m_AcmStreamHeader.cbDstLengthUsed = 0L;

     if(acmStreamConvert(m_hAcmStream, &m_AcmStreamHeader,
                                 ACM_STREAMCONVERTF_BLOCKALIGN) != MMSYSERR_NOERROR)
     {
          return false;
     }

     //Wait until the convert is on
     while((ACMSTREAMHEADER_STATUSF_DONE & m_AcmStreamHeader.fdwStatus) == 0)
     {
     }


     dwInputBytesUsed = m_AcmStreamHeader.cbSrcLengthUsed;
     dwOutputBytesUsed = m_AcmStreamHeader.cbDstLengthUsed;

     memcpy(pbOutputData, m_AcmStreamHeader.pbDst, dwOutputBytesUsed);

     return true;
}

/* Fin de CWaveConverter.cpp */
