/*==============================================================================
	CWaveOutBuffer.cpp
==============================================================================*/

#include "CWaveOutBuffer.hpp"

/*==============================================================================
	Constructeur
==============================================================================*/

CWaveOutBuffer::CWaveOutBuffer(long obj, WAVEWRITEOUTPROC proc, WAVEFORMATEX format, int nbBuffers, int sizeBuffer) :
	_hEvent(0), _bTerminateThread(0), _stWriteProc(proc),
	_hWaveOut(0), _wHeader(0), _wFormatEx(format),
	_pBuffer(0), _pObjData(obj),
	_iCountBuffers(nbBuffers), _iSizeBuffer(sizeBuffer), _iCurrentIndexBlock(0)
{

	_pThread				= new CThread(stThreadProc);

}

/*==============================================================================
	Destructeur
==============================================================================*/

CWaveOutBuffer::~CWaveOutBuffer()
{

	stop();
	delete _pThread;

}

/*==============================================================================
	start
==============================================================================*/

BOOL CWaveOutBuffer::start(void)
{

	MMRESULT	mRes;

	// Si le wav est en lecture, on le stoppe
	stop();

	mRes = waveOutOpen(&_hWaveOut, WAVE_MAPPER, &_wFormatEx, (DWORD)stWaveOutProc, (DWORD)this, CALLBACK_FUNCTION);
    if ( mRes != MMSYSERR_NOERROR )
        return FALSE;

	_iCurrentIndexBlock	= 0;
	_bTerminateThread	= FALSE;

	_hEvent				= CreateEvent(NULL, FALSE, FALSE, NULL);
	_pThread->start(this);

	return TRUE;

}

/*==============================================================================
	pause
==============================================================================*/

/*==============================================================================
	stop
==============================================================================*/

void CWaveOutBuffer::stop(void)
{

	if ( _pThread ) {
		_bTerminateThread = TRUE;
		if ( _hWaveOut )
			waveOutReset(_hWaveOut);
		_pThread->stop(INFINITE);
	}

}

/*==============================================================================
	prepareBuffer
==============================================================================*/

void CWaveOutBuffer::prepareBuffer(void)
{

	int			i;

	_wHeader			= new WAVEHDR[_iCountBuffers];
	_pBuffer			= new char*[_iCountBuffers];
	for ( i = 0; i < _iCountBuffers; i++ )
		_pBuffer[i]		= new char[_iSizeBuffer];

	for ( i = 0; i < _iCountBuffers; i++ ) {
		ZeroMemory(&_wHeader[i], sizeof(WAVEHDR));
		_wHeader[i].dwBufferLength	= _iSizeBuffer;
		_wHeader[i].lpData			= _pBuffer[i];
		waveOutPrepareHeader(_hWaveOut, &_wHeader[i], sizeof(WAVEHDR));
		writeBuffer(i);
    }

}

/*==============================================================================
	writeBuffer
==============================================================================*/

void CWaveOutBuffer::writeBuffer(int index)
{

	int			i, j, len;

	len			= _stWriteProc((LPVOID)_pObjData, _pBuffer[index]);

	// Si la longueur est invalide
	if ( len <= 0 || len > _iSizeBuffer ) {
		_bTerminateThread	= TRUE;
		return;
	}

	// Longueur et pointeur de la mmoire
	_wHeader[index].dwFlags			&= ~WHDR_DONE;
	_wHeader[index].dwBufferLength	= len;
	_wHeader[index].lpData			= _pBuffer[index];
    waveOutWrite(_hWaveOut, &_wHeader[index], sizeof(WAVEHDR));

}

/*==============================================================================
	freeBuffer
==============================================================================*/

void CWaveOutBuffer::freeBuffer(void)
{

	int			i;

	if ( _hWaveOut ) {
		waveOutReset(_hWaveOut);
		for ( i = 0; i < _iCountBuffers; i++ ) {
			while ( waveOutUnprepareHeader(_hWaveOut, &_wHeader[i], sizeof(WAVEHDR) ) == WAVERR_STILLPLAYING )
				Sleep(10);
		}
		waveOutClose(_hWaveOut);
		_hWaveOut	= 0;
	}

	// Supprime le buffer
	if ( _wHeader ) {
		delete [] _wHeader;
		_wHeader	= 0;
	}
	if ( _pBuffer ) {
		for ( i = 0; i < _iCountBuffers; i++ )
			delete [] _pBuffer[i];
		delete [] _pBuffer;
		_pBuffer	= 0;
	}

}

/*==============================================================================
	threadProc
==============================================================================*/

void CWaveOutBuffer::threadProc(void)
{

	int			i, j;

	prepareBuffer();

	while ( !_bTerminateThread ) {

		WaitForSingleObject(_hEvent, INFINITE);
		i = _iCurrentIndexBlock;

		if ( !_bTerminateThread && _wHeader[i].dwFlags & WHDR_DONE ) {

			_iCurrentIndexBlock++;
			if ( _iCurrentIndexBlock >= _iCountBuffers )
				_iCurrentIndexBlock = 0;
			writeBuffer(i);

		}

	}

	freeBuffer();

}

/*==============================================================================
	stThreadProc
==============================================================================*/

DWORD WINAPI CWaveOutBuffer::stThreadProc(PVOID param)
{

	CWaveOutBuffer		*pData;

	pData			= (CWaveOutBuffer *)param;
	if ( pData )
		pData->threadProc();
	return 0;

}

/*==============================================================================
	waveOutProc
==============================================================================*/

void CWaveOutBuffer::waveOutProc(UINT uMsg)
{

	if ( uMsg == WOM_DONE )
		SetEvent(_hEvent);

}

/*==============================================================================
	stWaveOutProc
==============================================================================*/

void CALLBACK CWaveOutBuffer::stWaveOutProc(HWAVEOUT hWo, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{

	CWaveOutBuffer		*pData;

	pData			= (CWaveOutBuffer *)dwInstance;
	if ( pData )
		pData->waveOutProc(uMsg);

}

/* Fin de CWaveOutBuffer.cpp */
